import argparse
import glob
import hashlib
import os
import json
import time
import uuid
import yaml

from autocoder_nano.agent.agent_define import get_subagent_define
from autocoder_nano.utils.file_utils import load_tokenizer, auto_count_file_extensions
from autocoder_nano.edit import run_edit
from autocoder_nano.helper import show_help
from autocoder_nano.index import extract_symbols
from autocoder_nano.rules import rules_from_active_files, get_rules_context
from autocoder_nano.core import AutoLLM
from autocoder_nano.actypes import *
from autocoder_nano.acmodels import BUILTIN_MODELS
from autocoder_nano.acrunner import (chat_command, new_index_command,
                                     execute_shell_command,
                                     generate_shell_command, revert, auto_command)
from autocoder_nano.utils.completer_utils import CommandCompleter
from autocoder_nano.version import __version__
from autocoder_nano.templates import create_actions
from autocoder_nano.utils.git_utils import (repo_init, commit_changes,
                                            get_uncommitted_changes, generate_commit_message)
from autocoder_nano.utils.sys_utils import default_exclude_dirs, default_exclude_files
from autocoder_nano.utils.printer_utils import (
    Printer, COLOR_WARNING, COLOR_ERROR, COLOR_SUCCESS, COLOR_SYSTEM, COLOR_INFO)
from autocoder_nano.utils.config_utils import (get_final_config, convert_yaml_config_to_str, convert_config_value,
                                               convert_yaml_to_config, get_last_yaml_file, prepare_chat_yaml)
from autocoder_nano.utils.lexer_utils import SimpleAutoCoderLexer
from autocoder_nano.utils.theme_utils import ThemeManager

from prompt_toolkit import prompt as _toolkit_prompt, PromptSession
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.formatted_text import FormattedText
from prompt_toolkit.history import InMemoryHistory
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.shortcuts import confirm, CompleteStyle
from prompt_toolkit.cursor_shapes import CursorShape
from rich.syntax import Syntax
from rich.text import Text


printer = Printer()
theme = ThemeManager()
console = printer.get_console()


project_root = os.getcwd()
base_persist_dir = os.path.join(project_root, ".auto-coder", "plugins", "chat-auto-coder")

# 命令行精简
# /rag/build, /index/build 合并为 /index, 移除 /rag/query, /index/query
# 移除 /mode
# /revert, /commit, 合并为 /git
# 移除 /shell, /editor, /context
commands = [
    "/auto", "/coding", "/chat",                                                       # 核心功能
    "/help", "/exit", "/models", "/conf", "/index", "/git", "/rules",                  # 辅助功能
    "/add_files", "/remove_files", "/list_files", "/exclude_dirs", "/exclude_files"        # 文件管理
]

memory = {
    "conversation": [],
    "current_files": {"files": [], "groups": {}},
    "conf": {
        "auto_merge": "editblock",
        "chat_model": "",
        "code_model": "",
    },
    "exclude_dirs": [],
    "mode": "normal",  # 新增mode字段,默认为normal模式
    "models": {},
    "theme": "cyberpunk"  # 新增theme字段，默认为cyberpunk
}


def get_all_file_names_in_project() -> List[str]:
    file_names = []
    final_exclude_dirs = default_exclude_dirs + memory.get("exclude_dirs", [])
    for root, dirs, files in os.walk(project_root, followlinks=True):
        dirs[:] = [d for d in dirs if d not in final_exclude_dirs]
        for file in files:
            if file not in default_exclude_files:
                file_names.append(file)
    return file_names


def get_all_file_in_project() -> List[str]:
    file_names = []
    final_exclude_dirs = default_exclude_dirs + memory.get("exclude_dirs", [])
    for root, dirs, files in os.walk(project_root, followlinks=True):
        dirs[:] = [d for d in dirs if d not in final_exclude_dirs]
        for file in files:
            if file not in default_exclude_files:
                file_names.append(os.path.join(root, file))
    return file_names


def get_all_dir_names_in_project() -> List[str]:
    dir_names = []
    final_exclude_dirs = default_exclude_dirs + memory.get("exclude_dirs", [])
    for root, dirs, files in os.walk(project_root, followlinks=True):
        dirs[:] = [d for d in dirs if d not in final_exclude_dirs]
        for _dir in dirs:
            dir_names.append(_dir)
    return dir_names


def get_all_file_in_project_with_dot() -> List[str]:
    file_names = []
    final_exclude_dirs = default_exclude_dirs + memory.get("exclude_dirs", [])
    for root, dirs, files in os.walk(project_root, followlinks=True):
        dirs[:] = [d for d in dirs if d not in final_exclude_dirs]
        for file in files:
            if file not in default_exclude_files:
                file_names.append(os.path.join(root, file).replace(project_root, "."))
    return file_names


def get_symbol_list() -> List[SymbolItem]:
    list_of_symbols = []
    index_file = os.path.join(project_root, ".auto-coder", "index.json")

    if os.path.exists(index_file):
        with open(index_file, "r") as file:
            index_data = json.load(file)
    else:
        index_data = {}

    for item in index_data.values():
        symbols_str = item["symbols"]
        module_name = item["module_name"]
        info1 = extract_symbols(symbols_str)
        for name in info1.classes:
            list_of_symbols.append(
                SymbolItem(symbol_name=name, symbol_type=SymbolType.CLASSES, file_name=module_name)
            )
        for name in info1.functions:
            list_of_symbols.append(
                SymbolItem(symbol_name=name, symbol_type=SymbolType.FUNCTIONS, file_name=module_name)
            )
        for name in info1.variables:
            list_of_symbols.append(
                SymbolItem(symbol_name=name, symbol_type=SymbolType.VARIABLES, file_name=module_name)
            )
    return list_of_symbols


def find_files_in_project(patterns: List[str]) -> List[str]:
    matched_files = []
    final_exclude_dirs = default_exclude_dirs + memory.get("exclude_dirs", [])

    for pattern in patterns:
        if "*" in pattern or "?" in pattern:
            for file_path in glob.glob(pattern, recursive=True):
                if os.path.isfile(file_path):
                    abs_path = os.path.abspath(file_path)
                    if not any(exclude_dir in abs_path.split(os.sep) for exclude_dir in final_exclude_dirs):
                        matched_files.append(abs_path)
        else:
            is_added = False
            # add files belongs to project
            for root, dirs, files in os.walk(project_root, followlinks=True):
                dirs[:] = [d for d in dirs if d not in final_exclude_dirs]
                filtered_files = [f for f in files if f not in default_exclude_files]
                if pattern in filtered_files:
                    matched_files.append(os.path.join(root, pattern))
                    is_added = True
                else:
                    for file in files:
                        _pattern = os.path.abspath(pattern)
                        if _pattern in os.path.join(root, file):
                            matched_files.append(os.path.join(root, file))
                            is_added = True
            # add files not belongs to project
            if not is_added:
                matched_files.append(pattern)
    return list(set(matched_files))


def save_memory():
    with open(os.path.join(base_persist_dir, "nano-memory.json"), "w") as fp:
        json_str = json.dumps(memory, indent=2, ensure_ascii=False)
        fp.write(json_str)
    load_memory()


def load_memory():
    global memory
    memory_path = os.path.join(base_persist_dir, "nano-memory.json")
    if os.path.exists(memory_path):
        try:
            with open(memory_path, "r") as f:
                memory = json.load(f)
        except json.JSONDecodeError as e:
            raise Exception(f"打开配置文件失败, 文件内容不是有效的JSON格式, 错误原因: {str(e)} , 文件位置: {memory_path}")


def get_memory():
    load_memory()
    return memory


completer = CommandCompleter(
    commands=commands,
    file_system_model=FileSystemModel(
        project_root=project_root,
        get_all_file_names_in_project=get_all_file_names_in_project,
        get_all_file_in_project=get_all_file_in_project,
        get_all_dir_names_in_project=get_all_dir_names_in_project,
        get_all_file_in_project_with_dot=get_all_file_in_project_with_dot,
        get_symbol_list=get_symbol_list
    ),
    memory_model=MemoryConfig(
        get_memory_func=get_memory,
        save_memory_func=save_memory
    )
)


def exclude_dirs(dir_names: List[str]):
    new_dirs = dir_names
    existing_dirs = memory.get("exclude_dirs", [])
    dirs_to_add = [d for d in new_dirs if d not in existing_dirs]

    if dirs_to_add:
        existing_dirs.extend(dirs_to_add)
        if "exclude_dirs" not in memory:
            memory["exclude_dirs"] = existing_dirs
        printer.print_text(Text(f"已添加排除目录: {dirs_to_add}", style=COLOR_SUCCESS))
        for d in dirs_to_add:
            exclude_files(f"regex://.*/{d}/*.")
        # exclude_files([f"regex://.*/{d}/*." for d in dirs_to_add])
    else:
        printer.print_text(Text(f"所有指定目录已在排除列表中. ", style=COLOR_SUCCESS))
    save_memory()
    completer.refresh_files()


def exclude_files(query: str):
    if "/list" in query:
        query = query.replace("/list", "", 1).strip()
        existing_file_patterns = memory.get("exclude_files", [])

        printer.print_table_compact(
            headers=["File Pattern"],
            data=[[file_pattern] for file_pattern in existing_file_patterns],
            title="Exclude Files"
        )
        return

    if "/drop" in query:
        query = query.replace("/drop", "", 1).strip()
        existing_file_patterns = memory.get("exclude_files", [])
        existing_file_patterns.remove(query.strip())
        memory["exclude_files"] = existing_file_patterns
        if query.startswith("regex://.*/") and query.endswith("/*."):
            existing_dirs_patterns = memory.get("exclude_dirs", [])
            dir_query = query.replace("regex://.*/", "", 1).replace("/*.", "", 1)
            if dir_query in existing_dirs_patterns:
                existing_dirs_patterns.remove(dir_query.strip())
        save_memory()
        completer.refresh_files()
        return

    new_file_patterns = query.strip().split(",")

    existing_file_patterns = memory.get("exclude_files", [])
    file_patterns_to_add = [f for f in new_file_patterns if f not in existing_file_patterns]

    for file_pattern in file_patterns_to_add:
        if not file_pattern.startswith("regex://"):
            raise

    if file_patterns_to_add:
        existing_file_patterns.extend(file_patterns_to_add)
        if "exclude_files" not in memory:
            memory["exclude_files"] = existing_file_patterns
        save_memory()
        printer.print_text(f"已添加排除文件: {file_patterns_to_add}. ", style=COLOR_SUCCESS)
    else:
        printer.print_text(f"所有指定文件已在排除列表中. ", style=COLOR_SUCCESS)


def init_project(project_type):
    if not project_type:
        printer.print_text(
            f"请指定项目类型。可选的项目类型包括：py|ts| 或文件扩展名(例如:.java,.scala), 多个扩展名逗号分隔.", style=COLOR_SUCCESS
        )
        return
    os.makedirs(os.path.join(project_root, "actions"), exist_ok=True)
    os.makedirs(os.path.join(project_root, ".auto-coder"), exist_ok=True)
    os.makedirs(os.path.join(project_root, ".auto-coder", "autocoderrules"), exist_ok=True)
    source_dir = os.path.abspath(project_root)
    create_actions(
        source_dir=source_dir,
        params={"project_type": project_type,
                "source_dir": source_dir},
    )

    repo_init(source_dir)
    with open(os.path.join(source_dir, ".gitignore"), "a") as f:
        f.write("\n.auto-coder/")
        f.write("\nactions/")
        f.write("\noutput.txt")

    printer.print_text(f"已在 {os.path.abspath(project_root)} 成功初始化 autocoder-nano 项目", style=COLOR_SUCCESS)
    return


def get_conversation_history() -> str:
    memory_dir = os.path.join(project_root, ".auto-coder", "memory")
    os.makedirs(memory_dir, exist_ok=True)
    memory_file = os.path.join(memory_dir, "chat_history.json")

    def error_message():
        printer.print_panel(Text("未找到可应用聊天记录.", style=COLOR_WARNING), title="Chat History", center=True)

    if not os.path.exists(memory_file):
        error_message()
        return ""

    with open(memory_file, "r") as f:
        chat_history = json.load(f)

    if not chat_history["ask_conversation"]:
        error_message()
        return ""

    conversations = chat_history["ask_conversation"]

    context = ""
    context += f"下面是我们的历史对话，参考我们的历史对话从而更好的理解需求和修改代码。\n\n<history>\n"
    for conv in conversations:
        if conv["role"] == "user":
            context += f"用户: {conv['content']}\n"
        elif conv["role"] == "assistant":
            context += f"你: {conv['content']}\n"
    context += "</history>\n"
    return context


def coding_command(query: str, llm: AutoLLM):
    is_apply = query.strip().startswith("/apply")
    if is_apply:
        query = query.replace("/apply", "", 1).strip()
    is_rules = False

    memory["conversation"].append({"role": "user", "content": query})
    conf = memory.get("conf", {})

    prepare_chat_yaml(project_root)  # 复制上一个序号的 yaml 文件, 生成一个新的聊天 yaml 文件
    latest_yaml_file = get_last_yaml_file(project_root)

    if latest_yaml_file:
        yaml_config = {
            "include_file": ["./base/base.yml"],
            "skip_build_index": conf.get("skip_build_index", "true") == "true",
            "skip_confirm": conf.get("skip_confirm", "true") == "true",
            "chat_model": conf.get("chat_model", ""),
            "code_model": conf.get("code_model", ""),
            "auto_merge": conf.get("auto_merge", "editblock"),
            "context": ""
        }

        for key, value in conf.items():
            converted_value = convert_config_value(key, value)
            if converted_value is not None:
                yaml_config[key] = converted_value

        yaml_config["urls"] = memory["current_files"]["files"]
        yaml_config["query"] = query

        if is_apply:
            yaml_config["context"] += get_conversation_history()

        if is_rules:
            yaml_config["context"] += get_rules_context(project_root)

        yaml_config["file"] = latest_yaml_file
        yaml_content = convert_yaml_config_to_str(yaml_config=yaml_config)
        execute_file = os.path.join(project_root, "actions", latest_yaml_file)
        with open(os.path.join(execute_file), "w") as f:
            f.write(yaml_content)
        args = convert_yaml_to_config(execute_file)

        run_edit(llm=llm, args=args)
    else:
        printer.print_text(f"创建新的 YAML 文件失败.", style=COLOR_WARNING)

    save_memory()
    completer.refresh_files()


def print_commit_info(commit_result: CommitResult, llm_commit_message: str):
    if commit_result.diffs:
        for file, diff in commit_result.diffs.items():
            printer.print_text(f"File: {file}", style=COLOR_SUCCESS)
            syntax = Syntax(diff, "diff", theme="monokai", line_numbers=True)
            printer.print_panel(syntax, title="File Diff", center=True)
        printer.print_table_compact(
            data=[
                ["提交说明", llm_commit_message],
                ["提交哈希", commit_result.commit_hash],
                ["提交信息", commit_result.commit_message],
                ["更改的文件", "\n".join(commit_result.changed_files) if commit_result.changed_files else "No files changed"]
            ],
            title="提交信息", headers=["属性", "值"], caption="(使用 /revert 撤销此提交)"
        )


def commit_info(query: str, llm: AutoLLM):
    repo_path = project_root
    prepare_chat_yaml(project_root)  # 复制上一个序号的 yaml 文件, 生成一个新的聊天 yaml 文件

    latest_yaml_file = get_last_yaml_file(project_root)
    execute_file = None

    if latest_yaml_file:
        try:
            execute_file = os.path.join(project_root, "actions", latest_yaml_file)
            conf = memory.get("conf", {})
            yaml_config = {
                "include_file": ["./base/base.yml"],
                "skip_build_index": conf.get("skip_build_index", "true") == "true",
                "skip_confirm": conf.get("skip_confirm", "true") == "true",
                "chat_model": conf.get("chat_model", ""),
                "code_model": conf.get("code_model", ""),
                "auto_merge": conf.get("auto_merge", "editblock"),
                "context": ""
            }
            for key, value in conf.items():
                converted_value = convert_config_value(key, value)
                if converted_value is not None:
                    yaml_config[key] = converted_value

            current_files = memory["current_files"]["files"]
            yaml_config["urls"] = current_files

            # 临时保存yaml文件，然后读取yaml文件，更新args
            temp_yaml = os.path.join(project_root, "actions", f"{uuid.uuid4()}.yml")
            try:
                with open(temp_yaml, "w", encoding="utf-8") as f:
                    f.write(convert_yaml_config_to_str(yaml_config=yaml_config))
                args = convert_yaml_to_config(temp_yaml)
            finally:
                if os.path.exists(temp_yaml):
                    os.remove(temp_yaml)

            # commit_message = ""
            commit_llm = llm
            commit_llm.setup_default_model_name(args.chat_model)
            printer.print_text(f"Commit 信息生成中...", style=COLOR_SUCCESS)

            try:
                uncommitted_changes = get_uncommitted_changes(repo_path)
                commit_message = generate_commit_message.with_llm(commit_llm).run(
                    uncommitted_changes
                )
                llm_to_commit_message = commit_message.output
                memory["conversation"].append({"role": "user", "content": llm_to_commit_message})
            except Exception as err:
                printer.print_text(f"Commit 信息生成失败: {err}", style=COLOR_ERROR)
                return

            yaml_config["query"] = llm_to_commit_message
            yaml_content = convert_yaml_config_to_str(yaml_config=yaml_config)
            with open(os.path.join(execute_file), "w", encoding="utf-8") as f:
                f.write(yaml_content)

            file_content = open(execute_file).read()
            md5 = hashlib.md5(file_content.encode("utf-8")).hexdigest()
            file_name = os.path.basename(execute_file)
            commit_result = commit_changes(repo_path, f"auto_coder_nano_{file_name}_{md5}")
            print_commit_info(commit_result=commit_result, llm_commit_message=llm_to_commit_message)
            if commit_message:
                printer.print_text(f"Commit 成功", style=COLOR_SUCCESS)
        except Exception as err:
            import traceback
            traceback.print_exc()
            printer.print_text(f"Commit 失败: {err}", style=COLOR_ERROR)
            if execute_file:
                os.remove(execute_file)


def git_command(git_args: List[str], llm: AutoLLM):
    """
    /git /commit
    /git /revert
    """
    if git_args[0] == "/commit":
        commit_info(query="", llm=llm)

    if git_args[0] == "/revert":
        revert(project_root=project_root)


def parse_args(input_args: Optional[List[str]] = None):
    parser = argparse.ArgumentParser(description="Auto-Coder Nano")

    parser.add_argument("--debug", action="store_true", help="开启 debug 模式")
    parser.add_argument("--quick", action="store_true", help="进入 auto-coder.nano 无需初始化系统")
    # 新增 --agent 参数
    parser.add_argument("--agent-define", type=str, help="定义要运行的Agent/SubAgent")
    parser.add_argument("--agent-query", type=str, help="指定Agent要执行的指令")
    parser.add_argument("--agent-new-session", action="store_true", help="是否以New Session启动Agent")
    parser.add_argument("--agent-model", action="store_true", help="是否以Web模式启动Agent")

    if input_args:
        _args = parser.parse_args(input_args)
    else:
        _args = parser.parse_args()

    return AutoCoderArgs(**vars(_args)), _args


def configure(conf: str, skip_print=False):
    parts = conf.split(None, 1)
    if len(parts) == 2 and parts[0] in ["/drop", "/unset", "/remove"]:
        key = parts[1].strip()
        if key in memory["conf"]:
            del memory["conf"][key]
            save_memory()
            printer.print_text(f"Deleted configuration: {key}", style=COLOR_SUCCESS)
        else:
            printer.print_text(f"Configuration not found: {key}", style=COLOR_WARNING)
    else:
        parts = conf.split(":", 1)
        if len(parts) != 2:
            printer.print_text(
                "Error: Invalid configuration format. Use 'key:value' or '/drop key'.", style=COLOR_ERROR)
            return
        key, value = parts
        key = key.strip()
        value = value.strip()
        if not value:
            printer.print_text("Error: Value cannot be empty. Use 'key:value'.", style=COLOR_ERROR)
            return
        memory["conf"][key] = value
        save_memory()
        if not skip_print:
            printer.print_text(f"Set {key} to {value}", style=COLOR_SUCCESS)


def configure_project_type() -> str:
    from prompt_toolkit.formatted_text import HTML
    from prompt_toolkit.shortcuts import print_formatted_text
    from prompt_toolkit.styles import Style
    from html import escape

    style = Style.from_dict(
        {
            "info": "#ansicyan",
            "warning": "#ansiyellow",
            "input-area": "#ansigreen",
            "header": "#ansibrightyellow bold",
        }
    )

    def print_info(text):
        print_formatted_text(
            HTML(f"<info>{escape(text)}</info>"), style=style)

    def print_warning(text):
        print_formatted_text(
            HTML(f"<warning>{escape(text)}</warning>"), style=style)

    def print_header(text):
        print_formatted_text(
            HTML(f"<header>{escape(text)}</header>"), style=style)

    print_header(f"\n=== 项目类型配置 ===\n")
    print_info("项目类型支持：")
    print_info("  - 语言后缀（例如：.py, .java, .ts）")
    print_info("  - 预定义类型：py（Python）, ts（TypeScript/JavaScript）")
    print_info("对于混合语言项目，使用逗号分隔的值.")
    print_info("示例：'.java,.scala' 或 '.py,.ts'")

    print_warning(f"如果留空, 默认为 'py'.\n")

    project_type = _toolkit_prompt("请输入项目类型：", default="py", style=style).strip()

    if project_type:
        configure(f"project_type:{project_type}", skip_print=True)
        configure("skip_build_index:false", skip_print=True)
        print_info(f"\n项目类型设置为： {project_type}")
    else:
        print_info(f"\n使用默认项目类型：py")

    print_warning(f"\n您可以稍后使用以下命令更改此设置:")
    print_warning("/conf project_type:<new_type>\n")

    return project_type


def initialize_system():
    printer.print_text(f"正在初始化系统...", style=COLOR_INFO)

    def _init_project():
        first_time = False
        if not os.path.exists(os.path.join(project_root, ".auto-coder")):
            first_time = True
            printer.print_text("当前目录未初始化为auto-coder项目.", style=COLOR_WARNING)
            init_choice = input(f"  是否现在初始化项目？(y/n): ").strip().lower()
            if init_choice == "y":
                try:
                    if first_time:  # 首次启动,配置项目类型
                        if not os.path.exists(base_persist_dir):
                            os.makedirs(base_persist_dir, exist_ok=True)
                            # printer.print_text("创建目录：{}".format(base_persist_dir), style=COLOR_SUCCESS)
                        count_file_ext = auto_count_file_extensions(project_root)
                        project_type = ".py"
                        if len(count_file_ext) > 0:
                            project_type = ",".join(count_file_ext.keys())
                        printer.print_text(f"项目类型设置为： {project_type}", style=COLOR_SUCCESS)
                        init_project(project_type)
                        printer.print_text(f"您可以稍后使用以下命令更改此设置:", style=COLOR_WARNING)
                        printer.print_text("/conf project_type:<new_type>", style=COLOR_WARNING)
                    printer.print_text("项目初始化成功.", style=COLOR_SUCCESS)
                except Exception as e:
                    printer.print_text(f"项目初始化失败, {str(e)}.", style=COLOR_ERROR)
                    exit(1)
            else:
                printer.print_text("退出而不初始化.", style=COLOR_WARNING)
                exit(1)

        printer.print_text("项目初始化完成.", style=COLOR_SUCCESS)

    _init_project()


def add_files(add_files_args: List[str]):
    if "groups" not in memory["current_files"]:
        memory["current_files"]["groups"] = {}
    if "groups_info" not in memory["current_files"]:
        memory["current_files"]["groups_info"] = {}
    if "current_groups" not in memory["current_files"]:
        memory["current_files"]["current_groups"] = []
    groups = memory["current_files"]["groups"]
    groups_info = memory["current_files"]["groups_info"]

    if not add_files_args:
        printer.print_panel(Text("请为 /add_files 命令提供参数.", style=COLOR_ERROR), title="错误", center=True)
        return

    if add_files_args[0] == "/refresh":  # 刷新
        completer.refresh_files()
        load_memory()
        printer.print_panel(Text("已刷新的文件列表.", style=COLOR_SUCCESS), title="文件刷新", center=True)
        return

    if add_files_args[0] == "/group":
        # 列出组
        if len(add_files_args) == 1 or (len(add_files_args) == 2 and add_files_args[1] == "list"):
            if not groups:
                printer.print_panel(Text("未定义任何文件组.", style=COLOR_WARNING), title="文件组", center=True)
            else:
                data_list = []
                for i, (group_name, files) in enumerate(groups.items()):
                    query_prefix = groups_info.get(group_name, {}).get("query_prefix", "")
                    is_active = ("✓" if group_name in memory["current_files"]["current_groups"] else "")
                    data_list.append([
                        group_name,
                        "\n".join([os.path.relpath(f, project_root) for f in files]),
                        query_prefix,
                        is_active
                    ])
                printer.print_table_compact(
                    data=data_list,
                    title="已定义文件组",
                    headers=["Group Name", "Files", "Query Prefix", "Active"]
                )
        # 重置活动组
        elif len(add_files_args) >= 2 and add_files_args[1] == "/reset":
            memory["current_files"]["current_groups"] = []
            printer.print_panel(
                Text("活动组名称已重置。如果你想清除活动文件，可使用命令 /remove_files /all .", style=COLOR_SUCCESS),
                title="活动组重置", center=True
            )
        # 新增组
        elif len(add_files_args) >= 3 and add_files_args[1] == "/add":
            group_name = add_files_args[2]
            groups[group_name] = memory["current_files"]["files"].copy()
            printer.print_panel(
                Text(f"已将当前文件添加到组 '{group_name}' .", style=COLOR_SUCCESS), title="新增组", center=True
            )
        # 删除组
        elif len(add_files_args) >= 3 and add_files_args[1] == "/drop":
            group_name = add_files_args[2]
            if group_name in groups:
                del memory["current_files"]["groups"][group_name]
                if group_name in groups_info:
                    del memory["current_files"]["groups_info"][group_name]
                if group_name in memory["current_files"]["current_groups"]:
                    memory["current_files"]["current_groups"].remove(group_name)
                printer.print_panel(
                    Text(f"已删除组 '{group_name}'.", style=COLOR_SUCCESS), title="删除组", center=True
                )
            else:
                printer.print_panel(
                    Text(f"组 '{group_name}' 未找到.", style=COLOR_ERROR), title="Error", center=True
                )
        # 支持多个组的合并，允许组名之间使用逗号或空格分隔
        elif len(add_files_args) >= 2:
            group_names = " ".join(add_files_args[1:]).replace(",", " ").split()
            merged_files = set()
            missing_groups = []
            for group_name in group_names:
                if group_name in groups:
                    merged_files.update(groups[group_name])
                else:
                    missing_groups.append(group_name)
            if missing_groups:
                printer.print_panel(
                    Text(f"未找到组: {', '.join(missing_groups)}", style=COLOR_ERROR), title="Error", center=True
                )
            if merged_files:
                memory["current_files"]["files"] = list(merged_files)
                memory["current_files"]["current_groups"] = [
                    name for name in group_names if name in groups
                ]
                printer.print_panel(
                    Text(f"合并来自组 {', '.join(group_names)} 的文件 .", style=COLOR_SUCCESS), title="文件合并", center=True
                )
                printer.print_table_compact(
                    data=[[os.path.relpath(f, project_root)] for f in memory["current_files"]["files"]],
                    title="当前文件",
                    headers=["File"]
                )
                printer.print_panel(
                    Text(f"当前组: {', '.join(memory['current_files']['current_groups'])}", style=COLOR_SUCCESS),
                    title="当前组", center=True
                )
            elif not missing_groups:
                printer.print_panel(
                    Text(f"指定组中没有文件.", style=COLOR_WARNING), title="未添加任何文件", center=True
                )

    else:
        existing_files = memory["current_files"]["files"]
        matched_files = find_files_in_project(add_files_args)

        files_to_add = [f for f in matched_files if f not in existing_files]
        if files_to_add:
            memory["current_files"]["files"].extend(files_to_add)
            printer.print_table_compact(
                data=[[os.path.relpath(f, project_root)] for f in files_to_add],
                title="新增文件",
                headers=["文件"]
            )
        else:
            printer.print_panel(
                Text(f"所有指定文件已存在于当前会话中，或者未找到匹配的文件.", style=COLOR_WARNING), title="未新增文件", center=True
            )

    completer.update_current_files(memory["current_files"]["files"])
    save_memory()


def remove_files(file_names: List[str]):
    if "/all" in file_names:
        memory["current_files"]["files"] = []
        memory["current_files"]["current_groups"] = []
        printer.print_panel("已移除所有文件", title="文件移除", center=True)
    else:
        removed_files = []
        for file in memory["current_files"]["files"]:
            if os.path.basename(file) in file_names:
                removed_files.append(file)
            elif file in file_names:
                removed_files.append(file)
        for file in removed_files:
            memory["current_files"]["files"].remove(file)

        if removed_files:
            printer.print_table_compact(
                data=[[os.path.relpath(f, project_root)] for f in removed_files],
                title="文件移除",
                headers=["File"]
            )
        else:
            printer.print_panel("未移除任何文件", title="未移除文件", border_style=COLOR_WARNING, center=True)
    completer.update_current_files(memory["current_files"]["files"])
    save_memory()


def list_files():
    current_files = memory["current_files"]["files"]

    if current_files:
        printer.print_table_compact(
            data=[[os.path.relpath(file, project_root)] for file in current_files],
            title="当前活跃文件",
            headers=["File"]
        )
    else:
        printer.print_panel("当前会话中无文件。", title="当前文件", center=True)


def print_conf(content: Dict[str, Any]):
    data_list = []
    for key in sorted(content.keys()):
        value = content[key]
        # Format value based on type
        if isinstance(value, (dict, list)):
            formatted_value = Text(json.dumps(value, indent=2), style=COLOR_WARNING)
        elif isinstance(value, bool):
            formatted_value = Text(str(value), style=COLOR_SUCCESS if value else COLOR_ERROR)
        elif isinstance(value, (int, float)):
            formatted_value = Text(str(value), style="bright_cyan")
        else:
            formatted_value = Text(str(value), style=COLOR_SUCCESS)
        data_list.append([str(key), formatted_value])
    printer.print_table_compact(
        data=data_list,
        title="Conf 配置",
        headers=["键", "值"],
        caption="使用 /conf <key>:<value> 修改这些设置"
    )


def print_models(content: Dict[str, Any]):
    data_list = []
    if content:
        for name in content:
            data_list.append([name, content[name].get("model", ""), content[name].get("base_url", "")])
    else:
        data_list.append(["", "", ""])
    printer.print_table_compact(
        headers=["Name", "Model Name", "Base URL"],
        title="模型列表",
        data=data_list,
        show_lines=True,
        expand=True
    )


def check_models(content: Dict[str, Any], llm: AutoLLM):
    def _check_single_llm(model):
        _start_time = time.monotonic()
        try:
            _response = llm.stream_chat_ai(
                conversations=[{"role": "user", "content": "ping, are you there?"}], model=model
            )
            for _chunk in _response:
                pass
            _latency = time.monotonic() - _start_time
            return True, _latency
        except Exception as e:
            return False, str(e)

    data_list = []
    if content:
        for name in content:
            attempt_ok, attempt_latency = _check_single_llm(name)
            if attempt_ok:
                data_list.append([name, Text("✓", style=COLOR_SUCCESS), f"{attempt_latency:.2f}s"])
            else:
                data_list.append([name, Text("✗", style=COLOR_ERROR), "-"])
    else:
        data_list.append(["", "", ""])
    printer.print_table_compact(
        headers=["模型名称", "状态", "延迟情况"],
        title="模型状态检测",
        data=data_list
    )


def manage_models(models_args, models_data, llm: AutoLLM):
    """
      /models /list - List all models (default + custom)
      /models /check - Check all models status (Latency)
      /models /add <name> <api_key> - Add model with simplified params
      /models /add_model name=xxx base_url=xxx api_key=xxxx model=xxxxx ... - Add model with custom params
      /models /remove <name> - Remove model by name
    """
    if models_args[0] == "/list":
        print_models(models_data)
    if models_args[0] == "/check":
        check_models(models_data, llm)
    if models_args[0] == "/add":
        m1, m2, m3, m4 = configure_project_model()
        printer.print_text("正在更新缓存...", style=COLOR_WARNING)
        memory["models"][m1] = {"base_url": m3, "api_key": m4, "model": m2}
        printer.print_text(f"供应商配置已成功完成！后续你可以使用 /models 命令, 查看, 新增和修改所有模型", style=COLOR_SUCCESS)
        printer.print_text(f"正在部署 {m1} 模型...", style=COLOR_SUCCESS)
        llm.setup_sub_client(m1,
                             memory["models"][m1]["api_key"],
                             memory["models"][m1]["base_url"],
                             memory["models"][m1]["model"])
    elif models_args[0] == "/add_model":
        add_model_args = models_args[1:]
        add_model_info = {item.split('=')[0]: item.split('=')[1] for item in add_model_args if item}
        mn = add_model_info["name"]
        printer.print_text(f"正在为 {mn} 更新缓存信息", style=COLOR_SUCCESS)
        if mn not in memory["models"]:
            memory["models"][mn] = {
                "base_url": add_model_info["base_url"],
                "api_key": add_model_info["api_key"],
                "model": add_model_info["model"]
            }
        else:
            printer.print_text(f"{mn} 已经存在, 请执行 /models /remove <name> 进行删除", style=COLOR_ERROR)
        printer.print_text(f"正在部署 {mn} 模型", style=COLOR_SUCCESS)
        llm.setup_sub_client(mn, add_model_info["api_key"], add_model_info["base_url"], add_model_info["model"])
    elif models_args[0] == "/remove":
        rmn = models_args[1]
        printer.print_text(f"正在清理 {rmn} 缓存信息", style=COLOR_SUCCESS)
        if rmn in memory["models"]:
            del memory["models"][rmn]
        printer.print_text(f"正在卸载 {rmn} 模型", style=COLOR_SUCCESS)
        if llm.get_sub_client(rmn):
            llm.remove_sub_client(rmn)
        if rmn == memory["conf"]["chat_model"]:
            printer.print_text(f"当前首选Chat模型 {rmn} 已被删除, 请立即 /conf chat_model: 调整", style=COLOR_WARNING)
        if rmn == memory["conf"]["code_model"]:
            printer.print_text(f"当前首选Code模型 {rmn} 已被删除, 请立即 /conf code_model: 调整", style=COLOR_WARNING)


def configure_project_model():
    from prompt_toolkit.formatted_text import HTML
    from prompt_toolkit.shortcuts import print_formatted_text
    from prompt_toolkit.styles import Style
    from html import escape

    style = Style.from_dict(
        {"info": "#ansicyan", "warning": "#ansiyellow", "input-area": "#ansigreen", "header": "#ansibrightyellow bold"}
    )

    def print_info(text):
        print_formatted_text(HTML(f"<info>{escape(text)}</info>"), style=style)

    def print_header(text):
        print_formatted_text(HTML(f"<header>{escape(text)}</header>"), style=style)

    def convert_models(builtin_models: dict[str: dict]):
        _result = {}
        for _key, _model_info in builtin_models.items():
            _model_id = _model_info["id"]
            _result[_model_id] = {
                "name": _key,
                "base_url": _model_info["base_url"],
                "model_name": _model_info["model_name"]
            }
        return _result

    default_model = convert_models(BUILTIN_MODELS)

    # 内置模型
    print_header(f"\n=== 正在配置项目模型 ===\n")
    print_info("Volcengine: https://www.volcengine.com/")
    print_info("OpenRouter: https://openrouter.ai/")
    print_info("iFlow: https://platform.iflow.cn/")
    print_info("")
    for key, model_info in BUILTIN_MODELS.items():
        model_id = model_info["id"]
        print_info(f"  {model_id}. {key}")
    print_info(f"  14. 其他模型")
    model_num = input(f"  请选择您想使用的模型供应商编号(1-14): ").strip().lower()

    if int(model_num) < 1 or int(model_num) > 14:
        printer.print_text("请选择 1-14", style=COLOR_ERROR)
        save_memory()
        exit(1)

    if model_num == "14":  # 只有选择"其他模型"才需要手动输入所有信息
        current_model = input(f"  设置你的首选模型别名(例如: deepseek-v3/r1, ark-deepseek-v3/r1): ").strip().lower()
        current_model_name = input(f"  请输入你使用模型的 Model Name: ").strip().lower()
        current_base_url = input(f"  请输入你使用模型的 Base URL: ").strip().lower()
        current_api_key = input(f"  请输入您的API密钥: ").strip()
        return current_model, current_model_name, current_base_url, current_api_key

    model_name_value = default_model[model_num].get("model_name", "")
    model_api_key = input(f"请输入您的 API 密钥：").strip()
    return (
        default_model[model_num]["name"],
        model_name_value,
        default_model[model_num]["base_url"],
        model_api_key
    )


def rules(query_args: List[str], llm: AutoLLM):
    """
    /rules 命令帮助:
    /rules /show            - 查看规则文件内容
    /rules /remove          - 删除规则文件
    /rules /analyze         - 分析当前文件，可选提供查询内容
    """
    args = get_final_config(project_root, memory, query="", delete_execute_file=True)
    rule_path = os.path.join(project_root, ".auto-coder", "RULES.md")

    if query_args[0] == "/show":  # /rules /show 参数检查
        if os.path.exists(rule_path):
            with open(rule_path, "r") as fp:
                printer.print_markdown(text=fp.read(), panel=True)
        else:
            printer.print_text(f"Rules 文件[{rule_path}]不存在", style=COLOR_WARNING)

    if query_args[0] == "/clear":
        if os.path.exists(rule_path):
            with open(rule_path, "w") as fp:
                fp.write("")
            printer.print_text(f"Rules 文件[{rule_path}]已重置", style=COLOR_WARNING)

    if query_args[0] == "/analyze":
        files = memory.get("current_files", {}).get("files", [])
        if not files:
            printer.print_text("当前无活跃文件用于生成 Rules", style=COLOR_WARNING)
            return

        rules_from_active_files(files=files, llm=llm, args=args)

    completer.refresh_files()


def is_old_version():
    # "0.1.26" 开始使用兼容 AutoCoder 的 chat_model, code_model 参数
    # 不再使用 current_chat_model 和 current_chat_model
    if 'current_chat_model' in memory['conf'] and 'current_code_model' in memory['conf']:
        printer.print_text(f"0.1.26 新增 chat_model, code_model 参数, 正在进行配置兼容性处理", style=COLOR_WARNING)
        memory['conf']['chat_model'] = memory['conf']['current_chat_model']
        memory['conf']['code_model'] = memory['conf']['current_code_model']
        del memory['conf']['current_chat_model']
        del memory['conf']['current_code_model']
    # "0.1.31" 在 .auto-coder 目录中新增 autocoderrules 目录
    rules_dir_path = os.path.join(project_root, ".auto-coder", "autocoderrules")
    if not os.path.exists(rules_dir_path):
        printer.print_text(f"0.1.31 .auto-coder 目录中新增 autocoderrules 目录, 正在进行配置兼容性处理", style=COLOR_WARNING)
        os.makedirs(rules_dir_path, exist_ok=True)
    # "0.4.1" 在 memory.json 中新增了 "theme": "cyberpunk"  # 新增theme字段，默认为cyberpunk
    if "theme" not in memory:
        memory["theme"] = "cyberpunk"


def main():
    _args, _raw_args = parse_args()
    _args.source_dir = project_root
    convert_yaml_to_config(_args)

    if not _raw_args.quick:
        initialize_system()

    try:
        load_memory()
        load_tokenizer()
        is_old_version()
        completer.update_current_files(memory["current_files"]["files"])
        completer.refresh_files()
    except Exception as e:
        printer.print_text(
            Text.assemble(
                ("初始化过程发生异常:", COLOR_ERROR),
                (f"{type(e).__name__}", COLOR_WARNING),
                (f" - {str(e)}", "")
            )
        )
        exit(1)

    if len(memory["models"]) == 0:
        _model_pass = input(f"  是否跳过模型配置(y/n): ").strip().lower()
        if _model_pass == "n":
            m1, m2, m3, m4 = configure_project_model()
            printer.print_text("正在更新缓存...", style=COLOR_WARNING)
            memory["conf"]["chat_model"] = m1
            memory["conf"]["code_model"] = m1
            memory["models"][m1] = {"base_url": m3, "api_key": m4, "model": m2}
            printer.print_text(f"供应商配置已成功完成！后续你可以使用 /models 命令, 查看, 新增和修改所有模型", style=COLOR_SUCCESS)
        else:
            printer.print_text("你已跳过模型配置,后续请使用 /models /add_model 添加模型...", style=COLOR_WARNING)
            printer.print_text("添加示例 /models /add_model name=& base_url=& api_key=& model=&", style=COLOR_WARNING)

    auto_llm = AutoLLM()  # 创建模型
    if len(memory["models"]) > 0:
        for _model_name in memory["models"]:
            printer.print_text(
                Text.assemble(
                    ("正在部署模型: ", COLOR_INFO),
                    (f"{_model_name}", COLOR_SUCCESS)
                ))
            auto_llm.setup_sub_client(_model_name,
                                      memory["models"][_model_name]["api_key"],
                                      memory["models"][_model_name]["base_url"],
                                      memory["models"][_model_name]["model"])

    printer.print_text("模型初始化完成.", style=COLOR_SUCCESS)

    if memory["conf"]["chat_model"] not in memory["models"].keys():
        printer.print_text("首选 Chat 模型与部署模型不一致, 请使用 /conf chat_model:& 设置", style=COLOR_ERROR)
    if memory["conf"]["code_model"] not in memory["models"].keys():
        printer.print_text("首选 Code 模型与部署模型不一致, 请使用 /conf code_model:& 设置", style=COLOR_ERROR)

    if _raw_args and _raw_args.agent_query:
        if _raw_args.agent_new_session:
            _agent_query = f'/new {_raw_args.agent_query}'
        else:
            _agent_query = f'{_raw_args.agent_query}'

        if _raw_args.agent_define:
            with open(_raw_args.agent_define, 'r', encoding='utf-8') as file:
                _agent_define = yaml.safe_load(file)
        else:
            _agent_define = get_subagent_define()
        try:
            auto_command(project_root=project_root, memory=memory, query=_agent_query, llm=auto_llm,
                         agent_define=_agent_define, is_web=_raw_args.agent_model)
        except Exception as e:
            printer.print_text(
                Text.assemble(
                    ("发生异常:", COLOR_ERROR),
                    (f"{type(e).__name__}", COLOR_WARNING),
                    (f" - {str(e)}", COLOR_SYSTEM))
            )
            if _raw_args.debug:
                import traceback
                traceback.print_exc()
        finally:
            return

    kb = KeyBindings()

    @kb.add("c-c")
    def _(event):
        # 如果在历史搜索模式中
        if event.app.layout.is_searching:
            event.app.current_buffer.history_search_text = None
            # 清除当前缓冲区
            event.app.current_buffer.reset()
        else:
            event.app.exit()

    @kb.add("c-t")  # 新增 Ctrl+T 切换主题
    def _(event):
        theme_list = list(theme.list_themes())
        current_theme = memory.get("theme", "cyberpunk")
        current_index = theme_list.index(current_theme) if current_theme in theme_list else 0
        next_index = (current_index + 1) % len(theme_list)
        next_theme = theme_list[next_index]

        memory["theme"] = next_theme
        save_memory()

        theme_name = theme.get_theme_name(next_theme)
        # 动态更新样式
        event.app.style = theme.get_theme(next_theme)

        printer.print_text(f"主题已切换至: {theme_name}", style=COLOR_SUCCESS)

    def get_bottom_toolbar():
        if "mode" not in memory:
            memory["mode"] = "normal"
        if "theme" not in memory:
            memory["theme"] = "cyberpunk"
        current_mode = memory["mode"]
        theme_name = theme.get_theme_name(memory["theme"])
        current_project = os.path.basename(project_root)
        current_model = memory["conf"]["chat_model"]
        return f" 当前项目: {current_project} | 主题: {theme_name} (ctl+t 切换) | 当前模型: {current_model}"

    current_theme_name = memory.get("theme", "cyberpunk")
    current_style = theme.get_theme(current_theme_name)
    session = PromptSession(
        # 输入行为
        # multiline=True,
        # prompt_continuation=lambda width, line_number, is_soft_wrap: " " * width,
        # 历史记录
        history=InMemoryHistory(),
        auto_suggest=AutoSuggestFromHistory(),
        enable_history_search=False,
        # 自动补全
        completer=completer,
        complete_while_typing=True,
        complete_in_thread=True,    # 在后台线程中补全
        complete_style=CompleteStyle.COLUMN,
        # 样式与外观
        cursor=CursorShape.BLINKING_BLOCK,
        lexer=SimpleAutoCoderLexer(),    # 实时语法高亮
        # 界面元素
        bottom_toolbar=get_bottom_toolbar,
        # 行为控制
        key_bindings=kb,
    )
    printer.print_key_value(
        {
            "AutoCoder Nano": f"v{__version__}",
            "Url": "https://github.com/w4n9H/autocoder-nano",
            "Help": "输入 /help 可以查看可用的命令(Ctrl + t 切换主题)."
        }
    )

    new_prompt = ""

    while True:
        try:
            prompt_message = [
                ("class:username", "coding"),
                ("class:at", "@"),
                ("class:host", "supa-nano"),
                ("class:colon", ":"),
                ("class:path", "~"),
                ("class:dollar", "$ "),
            ]

            if new_prompt:
                user_input = session.prompt(FormattedText(prompt_message), default=new_prompt, style=current_style)
            else:
                user_input = session.prompt(FormattedText(prompt_message), style=current_style)
            new_prompt = ""

            if "mode" not in memory:
                memory["mode"] = "normal"  # 默认为正常模式
            if memory["mode"] == "auto_detect" and user_input and not user_input.startswith("/"):
                shell_script = generate_shell_command(
                    project_root=project_root, memory=memory, input_text=user_input, llm=auto_llm)
                if confirm("是否要执行此脚本?"):
                    execute_shell_command(shell_script)
                else:
                    continue
            elif user_input.startswith("/add_files"):
                add_files_args = user_input[len("/add_files"):].strip().split()
                add_files(add_files_args)
            elif user_input.startswith("/remove_files"):
                file_names = user_input[len("/remove_files"):].strip().split(",")
                remove_files(file_names)
            elif user_input.startswith("/index"):
                index_args = user_input[len("/index"):].strip().split()
                if not index_args:
                    printer.print_text(Text("Please enter your request.", style=COLOR_ERROR))
                else:
                    new_index_command(index_args=index_args, project_root=project_root, memory=memory, llm=auto_llm)
            elif user_input.startswith("/list_files"):
                list_files()
            elif user_input.startswith("/conf"):
                conf = user_input[len("/conf"):].strip()
                if not conf:
                    print_conf(memory["conf"])
                else:
                    configure(conf)
            elif user_input.startswith("/git"):
                git_args = user_input[len("/git"):].strip().split()
                if not git_args:
                    printer.print_text(Text("Please enter your request.", style=COLOR_ERROR))
                else:
                    git_command(git_args, auto_llm)
            elif user_input.startswith("/rules"):
                query_args = user_input[len("/rules"):].strip().split()
                if not query_args:
                    printer.print_text("Please enter your request.", style=COLOR_WARNING)
                    continue
                rules(query_args=query_args, llm=auto_llm)
            elif user_input.startswith("/help"):
                show_help()
            elif user_input.startswith("/exit"):
                raise EOFError()
            elif user_input.startswith("/coding"):
                query = user_input[len("/coding"):].strip()
                if not query:
                    printer.print_text(Text("Please enter your request.", style=COLOR_ERROR))
                    continue
                coding_command(query=query, llm=auto_llm)
            elif user_input.startswith("/auto"):
                query = user_input[len("/auto"):].strip()
                if not query:
                    printer.print_text(Text("Please enter your request.", style=COLOR_ERROR))
                    continue
                auto_command(project_root=project_root, memory=memory, query=query, llm=auto_llm,
                             agent_define=get_subagent_define())
            elif user_input.startswith("/chat"):
                query = user_input[len("/chat"):].strip()
                if not query:
                    printer.print_text(Text("Please enter your request.", style=COLOR_ERROR))
                else:
                    chat_command(project_root=project_root, query=query, memory=memory, llm=auto_llm)
            elif user_input.startswith("/models"):
                models_args = user_input[len("/models"):].strip().split()
                if not models_args:
                    printer.print_text(Text("Please enter your request.", style=COLOR_ERROR))
                else:
                    manage_models(models_args, memory["models"], auto_llm)
            elif user_input.startswith("/exclude_dirs"):
                dir_names = user_input[len("/exclude_dirs"):].strip().split(",")
                exclude_dirs(dir_names)
            elif user_input.startswith("/exclude_files"):
                query = user_input[len("/exclude_files"):].strip()
                exclude_files(query)
            else:
                printer.print_text(Text("Please enter your request.", style=COLOR_ERROR))
        except KeyboardInterrupt:
            continue
        except EOFError:
            try:
                save_memory()
            except Exception as e:
                printer.print_text(
                    Text.assemble(
                        ("保存配置时发生异常:", COLOR_ERROR),
                        (f"{type(e).__name__}", COLOR_WARNING),
                        (f" - {str(e)}", "")
                    )
                )
            printer.print_text("退出 AutoCoder Nano...", style=COLOR_WARNING)
            break
        except Exception as e:
            printer.print_text(
                Text.assemble(
                    ("发生异常:", COLOR_ERROR),
                    (f"{type(e).__name__}", COLOR_WARNING),
                    (f" - {str(e)}", "")
                )
            )
            if _raw_args and _raw_args.debug:
                import traceback
                traceback.print_exc()


if __name__ == '__main__':
    main()



import os
import uuid
from collections import Counter

import pandas as pd
from pandas import DataFrame

from autocoder_nano.utils.db_context_utils import DuckDBLocalContext
from autocoder_nano.utils.printer_utils import Printer


printer = Printer()


class NoteBook:

    def __init__(self, project_root: str, database_name: str = "notes.db", keywords_filter_length: int = 3,
                 keywords_freq_top_n: int = 10):
        self.database_name = database_name

        self.notebook_dir = os.path.join(project_root, ".auto-coder", "notebook")
        os.makedirs(self.notebook_dir, exist_ok=True)

        import warnings
        warnings.filterwarnings("ignore", category=UserWarning)
        # from cutword import NER
        from cutword import Cutter
        self.cutter = Cutter(want_long_word=True)
        # self.ner = NER()

        self.keywords_filter_length = keywords_filter_length
        self.keywords_freq_top_n = keywords_freq_top_n

        if self.database_name == ":memory:":
            raise Exception("AgenticNoteBook 不支持 :memory: 用法,请指定数据库名称")

        self.database_path = os.path.join(self.notebook_dir, self.database_name)

        if not os.path.exists(self.database_path):
            self._initialize()

    @classmethod
    def class_name(cls) -> str:
        return "DuckDBVectorStore"

    def _initialize(self) -> None:
        """
        user_id
        身份标识：唯一确定笔记所有者
        数据隔离：实现用户间的数据隔离
        扩展基础：支持多租户、个性化、分析等功能
        安全边界：定义数据访问权限边界
        context
        情境智能：使笔记具有"记忆"其产生背景的能力
        精准检索：大幅提高搜索结果的相关性
        矛盾解决：帮助理解表面矛盾背后的情境差异
        行为预测：基于历史上下文预测用户需求
        个性适配：根据上下文调整Agent的响应风格
        """
        _query = """
        -- 主笔记表
        CREATE TABLE IF NOT EXISTS notes (
            id INTEGER PRIMARY KEY DEFAULT nextval('note_id_seq'),
            user_id VARCHAR NOT NULL,
            created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
            content TEXT NOT NULL,
            context TEXT,          -- 简化的上下文信息
            is_conflicted BOOLEAN DEFAULT FALSE
        );
        -- 关键词索引表
        CREATE TABLE IF NOT EXISTS keywords (
            keyword VARCHAR NOT NULL,
            note_id BIGINT NOT NULL,
            PRIMARY KEY (keyword, note_id)
        );
        -- 实体表 (简化的实体存储)
        CREATE TABLE IF NOT EXISTS entities (
            note_id BIGINT NOT NULL,
            entity_type VARCHAR NOT NULL,
            entity_value VARCHAR NOT NULL,
            PRIMARY KEY (note_id, entity_type, entity_value)
        );
        -- 新增：全文检索表
        CREATE TABLE IF NOT EXISTS full_text (
            document_identifier VARCHAR,
            tokenized_content TEXT,
            note_id INTEGER NOT NULL
        );
        """
        with DuckDBLocalContext(self.database_path) as _conn:
            _conn.execute("CREATE SEQUENCE IF NOT EXISTS note_id_seq;")
            _conn.execute(_query)
            # 创建索引加速搜索
            _conn.execute("CREATE INDEX idx_keywords ON keywords(keyword)")
            _conn.execute("CREATE INDEX idx_entities ON entities(entity_value)")

    def _extract_tokens(self, text) -> list[str]:
        """ 分词 """
        return self.cutter.cutword(text)

    def _calculate_tf(self, keywords: list[str]) -> Counter:
        """ 计算词频(TF) """
        return Counter(filter(lambda x: len(x) >= self.keywords_filter_length, keywords))

    def _filter_keywords(self, keyword_freq: Counter) -> list[str]:
        """ 分词 """
        return [kw for kw, _ in keyword_freq.most_common(self.keywords_freq_top_n)]

    # def _extract_entities(self, text):
    #     doc = self.ner.predict(text, return_words=False)
    #     entities = [(ent.entity, ent.ner_type_zh) for ent in doc[0] if doc[0]]
    #     return entities

    def extract_features(self, text: str):
        """提取关键词和实体"""
        # 去长度>3的词组统计词频,并取前5的词组作为keywords
        keywords = self._extract_tokens(text)
        keyword_freq = self._calculate_tf(keywords)
        top_keywords: list[str] = self._filter_keywords(keyword_freq)

        # 提取实体,关键词
        return {
            "keywords": " ".join(keywords),
            # "entities": self._extract_entities(text),
            "top_keywords": top_keywords,
            "tf_keywords": keyword_freq
        }

    def add_note(self, user_id, text, context=None) -> int:
        # 提取文本特征
        features = self.extract_features(text)

        _insert_query = """
        INSERT INTO notes (user_id, content, context) VALUES (?, ?, ?) RETURNING id
        """
        query_params = [user_id, text, context]
        with DuckDBLocalContext(self.database_path) as _conn:
            _note_id = _conn.execute(_insert_query, query_params).fetchone()[0]

        # 索引关键词
        self._index_keywords(_note_id, features["top_keywords"])
        # 全文索引
        self._index_full_text(_note_id, features["keywords"])
        # 索引实体
        # self._index_entities(_note_id, features["entities"])
        # 重建全文索引
        self._rebuild_full_text_index()

        return _note_id

    def _rebuild_full_text_index(self):
        with DuckDBLocalContext(self.database_path) as _conn:
            try:
                _conn.execute("PRAGMA drop_fts_index('full_text');")
            except:
                pass
            finally:
                _conn.execute("PRAGMA create_fts_index('full_text', 'document_identifier', 'tokenized_content');")

    def _index_full_text(self, note_id, keywords: str):
        """ 全文索引 """
        with DuckDBLocalContext(self.database_path) as _conn:
            _insert_query = """INSERT INTO full_text (document_identifier, tokenized_content, note_id) 
            VALUES (?, ?, ?)"""
            query_params = [str(uuid.uuid4()), keywords, note_id]
            _conn.execute(_insert_query, query_params).fetchall()

    def _index_keywords(self, note_id, keywords):
        """ 索引 keywords """
        with DuckDBLocalContext(self.database_path) as _conn:
            for keyword in keywords:
                _insert_query = """INSERT OR IGNORE INTO keywords (keyword, note_id) VALUES (?, ?)"""
                query_params = [keyword, note_id]
                _conn.execute(_insert_query, query_params).fetchall()

    def _index_entities(self, note_id, entities):
        """ 索引实体 """
        with DuckDBLocalContext(self.database_path) as _conn:
            for ent in entities:
                _insert_query = """INSERT OR IGNORE INTO entities (note_id, entity_type, entity_value) 
                VALUES (?, ?, ?)"""
                query_params = [note_id, ent[1], ent[0]]
                _conn.execute(_insert_query, query_params).fetchall()

    def _keyword_search(self, user_id, query, limit):
        """ 关键词搜索 """
        query_words = self._extract_tokens(query)
        _query_sql = f"""
        SELECT n.id, n.content, n.created_at
        FROM notes n
        JOIN keywords k ON n.id = k.note_id
        WHERE n.user_id = ? 
          AND k.keyword IN ({','.join(['?']*len(query_words))})
        GROUP BY n.id, n.content, n.created_at
        ORDER BY COUNT(k.keyword) DESC
        LIMIT ?
        """
        query_params = [user_id, *query_words, limit]
        with DuckDBLocalContext(self.database_path) as _conn:
            return _conn.execute(_query_sql, query_params).fetchdf()

    # def _entity_search(self, user_id, query, limit):
    #     """ 实体搜索 """
    #     query_entities = [i[0] for i in self._extract_entities(query)]
    #     if query_entities:
    #         query_entities_join = f"{','.join(['?']*len(query_entities))}"
    #     else:
    #         query_entities_join = ""
    #     print(query_entities_join)
    #     _query_sql = f"""
    #     SELECT n.id, n.content, n.created_at
    #     FROM notes n
    #     JOIN entities e ON n.id = e.note_id
    #     WHERE n.user_id = ?
    #       AND e.entity_value IN (?)
    #     GROUP BY n.id, n.content, n.created_at
    #     ORDER BY COUNT(e.entity_value) DESC
    #     LIMIT ?
    #     """
    #     query_params = [user_id, query_entities_join, limit]
    #     with DuckDBLocalContext(self.database_path) as _conn:
    #         return _conn.execute(_query_sql, query_params).fetchdf()

    def _fulltext_search(self, user_id, query, limit):
        """ 全文搜索 """
        query_words = self._extract_tokens(query)
        query_words_join = " ".join(query_words)
        _query_sql = """
        SELECT n.id, n.content, n.created_at
        FROM notes n
        JOIN (
            SELECT note_id, score
            FROM (
                SELECT *, fts_main_full_text.match_bm25(
                    document_identifier, ?
                ) AS score
                FROM full_text
            ) sq
            WHERE score IS NOT NULL
            ORDER BY score DESC
        ) f ON n.id = f.note_id
        WHERE n.user_id = ?
        GROUP BY n.id, n.content, n.created_at
        LIMIT ?
        """
        query_params = [query_words_join, user_id, limit]
        with DuckDBLocalContext(self.database_path) as _conn:
            return _conn.execute(_query_sql, query_params).fetchdf()

    def search_by_query(self, user_id, query, limit=5) -> DataFrame:
        """ 统一搜索接口 """
        fs_df = self._keyword_search(user_id, query, limit)
        # es_df = self._entity_search(user_id, query, limit)
        fts_df = self._fulltext_search(user_id, query, limit)
        combined = pd.concat([fs_df, fts_df], axis=0, ignore_index=True)
        result = combined.drop_duplicates()
        printer.print_key_value(
            items={
                "关键词检索": f"{len(fs_df)} 条",
                # "实体检索": f"{len(es_df)} 条",
                "全文检索": f"{len(fts_df)} 条",
                "整体去重后数据条数": f"{len(result)} 条",
            },
            title="Notebook 检索结果"
        )
        return result


if __name__ == '__main__':
    source_dir = "/Users/moofs/Code/autocoder-nano"
    note = NoteBook(source_dir)
    # note.add_note(
    #     user_id="agentic",
    #     text="AutoCoder Nano 是一款轻量级的编码助手, 利用大型语言模型（LLMs）帮助开发者编写, 理解和修改代码",
    #     context="编码助手"
    # )
    # note.add_note(
    #     user_id="agentic",
    #     text="它提供了一个交互式命令行界面，支持在软件开发场景中与LLMs互动，具备代码生成, 文件管理和上下文代码理解等功能。",
    #     context="编码助手"
    # )
    # note.add_note(
    #     user_id="agentic",
    #     text="AutoCoder Nano 是 Auto-Coder 生态系统的简化版本，设计轻量且依赖极少。它旨在通过提供增强AI功能的命令行界面，弥合自然语言指令与代码修改之间的鸿沟。",
    #     context="编码助手"
    # )
    # note.add_note(
    #     user_id="agentic",
    #     text="我在2025年5月20日新增了rag功能",
    #     context="编码助手"
    # )

    # print(note.search_by_query("agentic", "AutoCoder Nano 是什么", search_type="KEYWORD"))
    # print(note.search_by_query("agentic", "我2025年5月20日更新了什么", search_type="ENTITY"))
    print(note.search_by_query("agentic", "AutoCoder Nano 是什么", search_type="FULLTEXT"))



from autocoder_nano.context.notebook import NoteBook
from autocoder_nano.context.context_prune import ConversationsPruner, ContentPruner
from autocoder_nano.context.context_manager import (
    get_context_manager, get_context_manager_config, reset_context_manager, ContextManagerConfig
)


def record_memory(project_root: str, user_id: str, content: str, context: str = None) -> int:
    """
    记录重要记忆到笔记系统
    Args:
        project_root
        user_id: 身份标识，诸如
        content: 需要记录的核心信息
        context: 记忆上下文(可选)
    Returns:
        新建笔记的ID
    """
    note = NoteBook(project_root=project_root)
    return note.add_note(user_id=user_id, text=content, context=context)


def recall_memory(project_root: str, user_id: str, query: str, limit: int = 5) -> str:
    """
    从记忆中检索相关信息
    Args:
        project_root:
        user_id: Agent身份标识
        query: 你的问题
        limit: 返回结果数量
    Returns:
        相关笔记的DataFrame(id, content, created_at)
    """
    note = NoteBook(project_root=project_root)
    df = note.search_by_query(user_id=user_id, query=query, limit=limit)
    results = []
    if not df.empty:
        results = []
        for _, row in df.iterrows():
            # 格式化时间戳
            created_at = row['created_at'].strftime("%Y-%m-%d %H:%M:%S")
            results.append(f"{created_at}:\n{row['content']}\n{'-' * 40}")
    return "\n".join(results)


__all__ = ["record_memory", "recall_memory", "ConversationsPruner", "ContentPruner", "ContextManagerConfig",
           "get_context_manager", "get_context_manager_config",
           "reset_context_manager"]


import os
import sys
import time
from prompt_toolkit import Application
from prompt_toolkit.buffer import Buffer
from prompt_toolkit.layout.containers import HSplit, Window, ConditionalContainer
from prompt_toolkit.layout.controls import BufferControl, FormattedTextControl
from prompt_toolkit.layout.layout import Layout
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.lexers import Lexer
from prompt_toolkit.styles import Style
from prompt_toolkit.widgets import TextArea
from prompt_toolkit.history import FileHistory
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.filters import Condition
from prompt_toolkit.layout.dimension import D
from prompt_toolkit.search import start_search
from prompt_toolkit.document import Document

# 高亮类型常量
HL_NORMAL = 0
HL_NONPRINT = 1
HL_COMMENT = 2
HL_MLCOMMENT = 3
HL_KEYWORD1 = 4
HL_KEYWORD2 = 5
HL_STRING = 6
HL_NUMBER = 7
HL_MATCH = 8

# 高亮标志
HL_HIGHLIGHT_STRINGS = 1 << 0
HL_HIGHLIGHT_NUMBERS = 1 << 1

# 特殊键值
CTRL_S = 'c-s'
CTRL_Q = 'c-q'
CTRL_F = 'c-f'
ESC = 'escape'
ENTER = 'enter'


class EditorSyntax:
    def __init__(self, filematch, keywords, singleline_comment_start,
                 multiline_comment_start, multiline_comment_end, flags):
        self.filematch = filematch
        self.keywords = keywords
        self.singleline_comment_start = singleline_comment_start
        self.multiline_comment_start = multiline_comment_start
        self.multiline_comment_end = multiline_comment_end
        self.flags = flags


class NanoLexer(Lexer):
    def __init__(self, syntax, search_query=None):
        self.syntax = syntax
        self.multiline_comment_open = False
        self.search_query = search_query

    def lex_document(self, document):
        # 如果没有语法定义，使用普通词法分析
        if not self.syntax:
            return self.default_lex_document(document)

        lines = document.lines
        prev_has_open_comment = self.multiline_comment_open

        def get_line(lineno):
            if lineno < 0 or lineno >= len(lines):
                return []

            line = lines[lineno]
            tokens = []
            in_string = None
            in_comment = prev_has_open_comment if lineno == 0 else (
                    lineno > 0 and get_line(lineno - 1)[-1][0] == 'comment')
            prev_sep = True
            i = 0

            # 处理搜索高亮
            search_highlight_positions = []
            if self.search_query:
                start = 0
                query_len = len(self.search_query)
                while start <= len(line):
                    pos = line.find(self.search_query, start)
                    if pos == -1:
                        break
                    search_highlight_positions.append((pos, pos + query_len))
                    start = pos + 1

            while i < len(line):
                # 检查是否在搜索高亮范围内
                in_search_highlight = False
                for start, end in search_highlight_positions:
                    if start <= i < end:
                        in_search_highlight = True
                        break

                # 处理搜索高亮
                if in_search_highlight:
                    j = min(end, len(line))
                    tokens.append(('class:search_match', line[i:j]))
                    i = j
                    prev_sep = True
                    continue

                char = line[i]

                # 处理单行注释
                if (prev_sep and i + len(self.syntax.singleline_comment_start) <= len(line) and
                        line[i:i + len(self.syntax.singleline_comment_start)] == self.syntax.singleline_comment_start):
                    tokens.append(('class:comment', line[i:]))
                    break

                # 处理多行注释
                if in_comment:
                    end_pos = line.find(self.syntax.multiline_comment_end, i)
                    if end_pos == -1:
                        tokens.append(('class:comment', line[i:]))
                        break
                    else:
                        tokens.append(('class:comment', line[i:end_pos + len(self.syntax.multiline_comment_end)]))
                        i = end_pos + len(self.syntax.multiline_comment_end)
                        in_comment = False
                        prev_sep = True
                        continue
                elif (i + len(self.syntax.multiline_comment_start) <= len(line) and
                      line[i:i + len(self.syntax.multiline_comment_start)] == self.syntax.multiline_comment_start):
                    end_pos = line.find(self.syntax.multiline_comment_end, i + len(self.syntax.multiline_comment_start))
                    if end_pos == -1:
                        tokens.append(('class:comment', line[i:]))
                        in_comment = True
                        break
                    else:
                        tokens.append(('class:comment', line[i:end_pos + len(self.syntax.multiline_comment_end)]))
                        i = end_pos + len(self.syntax.multiline_comment_end)
                        prev_sep = True
                        continue

                # 处理字符串
                if in_string:
                    end_pos = line.find(in_string, i)
                    if end_pos == -1:
                        tokens.append(('class:string', line[i:]))
                        break
                    else:
                        tokens.append(('class:string', line[i:end_pos + 1]))
                        i = end_pos + 1
                        in_string = None
                        prev_sep = False
                        continue
                else:
                    if char == '"' or char == "'":
                        end_pos = line.find(char, i + 1)
                        if end_pos == -1:
                            tokens.append(('class:string', line[i:]))
                            in_string = char
                            break
                        else:
                            tokens.append(('class:string', line[i:end_pos + 1]))
                            i = end_pos + 1
                            prev_sep = False
                            continue

                # 处理数字
                if char.isdigit() and (prev_sep or (i > 0 and tokens and tokens[-1][0] == 'number')):
                    j = i
                    while j < len(line) and (line[j].isdigit() or line[j] in '.eE-+'):
                        j += 1
                    tokens.append(('class:number', line[i:j]))
                    i = j
                    prev_sep = False
                    continue

                # 处理关键字
                if prev_sep:
                    found_keyword = False
                    for keyword in self.syntax.keywords:
                        kw = keyword.rstrip('|')
                        kw2 = keyword.endswith('|')
                        kw_len = len(kw)

                        if (i + kw_len <= len(line) and
                                line[i:i + kw_len] == kw and
                                (i + kw_len == len(line) or not line[i + kw_len].isalnum())):
                            token_type = 'class:keyword2' if kw2 else 'class:keyword1'
                            tokens.append((token_type, line[i:i + kw_len]))
                            i += kw_len
                            prev_sep = False
                            found_keyword = True
                            break

                    if found_keyword:
                        continue

                # 非关键字文本
                j = i
                while j < len(line):
                    if (line[j] in ' \t\n' or
                            (self.syntax.singleline_comment_start and
                             j + len(self.syntax.singleline_comment_start) <= len(line) and
                             line[j:j + len(
                                 self.syntax.singleline_comment_start)] == self.syntax.singleline_comment_start) or
                            (self.syntax.multiline_comment_start and
                             j + len(self.syntax.multiline_comment_start) <= len(line) and
                             line[
                             j:j + len(self.syntax.multiline_comment_start)] == self.syntax.multiline_comment_start)):
                        break
                    j += 1

                tokens.append(('', line[i:j]))
                i = j
                prev_sep = not line[i - 1].isalnum() if i > 0 else True

            # 更新多行注释状态
            if lineno == len(lines) - 1:
                self.multiline_comment_open = in_comment

            return tokens

        return get_line

    def default_lex_document(self, document):
        """默认词法分析 - 仅处理搜索高亮"""
        lines = document.lines

        def get_line(lineno):
            if lineno < 0 or lineno >= len(lines):
                return []

            line = lines[lineno]
            tokens = []
            i = 0

            # 处理搜索高亮
            search_highlight_positions = []
            if self.search_query:
                start = 0
                query_len = len(self.search_query)
                while start <= len(line):
                    pos = line.find(self.search_query, start)
                    if pos == -1:
                        break
                    search_highlight_positions.append((pos, pos + query_len))
                    start = pos + 1

            while i < len(line):
                # 检查是否在搜索高亮范围内
                in_search_highlight = False
                for start, end in search_highlight_positions:
                    if start <= i < end:
                        in_search_highlight = True
                        break

                if in_search_highlight:
                    j = min(end, len(line))
                    tokens.append(('class:search_match', line[i:j]))
                    i = j
                else:
                    j = i + 1
                    tokens.append(('', line[i:j]))
                    i = j

            return tokens

        return get_line


class NanoEditor:
    def __init__(self, filename):
        self.filename = filename
        self.dirty = False
        self.statusmsg = ""
        self.statusmsg_time = 0
        self.syntax = None
        self.search_mode = False
        self.search_query = ""
        self.search_history = []
        self.search_results = []
        self.current_search_index = -1
        self.last_search_query = ""

        # 初始化语法高亮数据库
        c_extensions = [".c", ".h", ".cpp", ".hpp", ".cc"]
        c_keywords = [
            "auto", "break", "case", "continue", "default", "do", "else", "enum",
            "extern", "for", "goto", "if", "register", "return", "sizeof", "static",
            "struct", "switch", "typedef", "union", "volatile", "while", "NULL",
            "alignas", "alignof", "and", "and_eq", "asm", "bitand", "bitor", "class",
            "compl", "constexpr", "const_cast", "deltype", "delete", "dynamic_cast",
            "explicit", "export", "false", "friend", "inline", "mutable", "namespace",
            "new", "noexcept", "not", "not_eq", "nullptr", "operator", "or", "or_eq",
            "private", "protected", "public", "reinterpret_cast", "static_assert",
            "static_cast", "template", "this", "thread_local", "throw", "true", "try",
            "typeid", "typename", "virtual", "xor", "xor_eq",
            "int|", "long|", "double|", "float|", "char|", "unsigned|", "signed|",
            "void|", "short|", "auto|", "const|", "bool|"
        ]

        self.HLDB = [
            EditorSyntax(
                c_extensions,
                c_keywords,
                "//",
                "/*",
                "*/",
                HL_HIGHLIGHT_STRINGS | HL_HIGHLIGHT_NUMBERS
            )
        ]

        # 选择语法高亮
        self.select_syntax_highlight()

        # 创建文本区域
        self.text_area = TextArea(
            text=self.load_file(),
            lexer=NanoLexer(self.syntax) if self.syntax else None,
            scrollbar=True,
            line_numbers=True,
            multiline=True,
            wrap_lines=False,
            history=FileHistory('.nano_history'),
            auto_suggest=AutoSuggestFromHistory(),
            style='class:editor'
        )

        # 搜索输入框
        self.search_input = TextArea(
            height=1,
            prompt="搜索: ",
            style='class:search',
            multiline=False,
            wrap_lines=False,
            history=FileHistory('.nano_search_history'),
            auto_suggest=AutoSuggestFromHistory(),
        )

        # 状态栏
        self.status_bar = Window(
            content=FormattedTextControl(self.get_status_text),
            height=1,
            style="class:status"
        )

        # 消息栏
        self.message_bar = Window(
            content=FormattedTextControl(self.get_message_text),
            height=1,
            style="class:message"
        )

        # 创建布局
        self.root_container = HSplit([
            self.text_area,
            self.status_bar,
            ConditionalContainer(
                content=self.search_input,
                filter=Condition(lambda: self.search_mode)
            ),
            self.message_bar
        ])

        # 创建按键绑定
        self.bindings = KeyBindings()
        self.setup_key_bindings()

        # 创建样式
        self.style = Style([
            ('status', 'bg:#0055ff fg:#ffffff'),
            ('message', 'bg:#00aa00 fg:#ffffff'),
            ('search', 'bg:#222222 fg:#ffffff'),
            ('keyword1', 'fg:#ff5555 bold'),
            ('keyword2', 'fg:#5555ff bold'),
            ('string', 'fg:#00aa00'),
            ('number', 'fg:#aa00aa'),
            ('comment', 'fg:#888888 italic'),
            ('search_match', 'bg:#555500 fg:#ffffff'),
        ])

        # 创建应用
        self.application = Application(
            layout=Layout(self.root_container, focused_element=self.text_area),
            key_bindings=self.bindings,
            mouse_support=True,
            style=self.style,
            full_screen=True
        )

    def select_syntax_highlight(self):
        """根据文件名选择语法高亮方案"""
        if not self.filename:
            return

        for syntax in self.HLDB:
            for pattern in syntax.filematch:
                if pattern.startswith('.'):
                    # 文件扩展名匹配
                    if self.filename.endswith(pattern):
                        self.syntax = syntax
                        return
                else:
                    # 文件名包含匹配
                    if pattern in self.filename:
                        self.syntax = syntax
                        return

    def load_file(self):
        """加载文件内容"""
        try:
            with open(self.filename, 'r') as f:
                return f.read()
        except FileNotFoundError:
            return ""
        except Exception as e:
            return f"Error opening file: {e}"

    def save_file(self):
        """保存文件"""
        try:
            with open(self.filename, 'w') as f:
                f.write(self.text_area.text)
            self.dirty = False
            self.set_status_message(f"文件保存成功, {len(self.text_area.text)} 字节写入磁盘.")
            return True
        except Exception as e:
            self.set_status_message(f"文件保存失败! I/O 错误: {str(e)}.")
            return False

    def set_status_message(self, message, *args):
        """设置状态消息"""
        self.statusmsg = message % args if args else message
        self.statusmsg_time = time.time()

    def get_status_text(self):
        """获取状态栏文本"""
        status = f"{os.path.basename(self.filename)} - {len(self.text_area.text.splitlines())} 行"
        if self.dirty:
            status += " (已修改)"
        return status

    def get_message_text(self):
        """获取消息栏文本"""
        if self.statusmsg and time.time() - self.statusmsg_time < 5:
            return self.statusmsg

        if self.search_mode:
            if self.search_results:
                return f"找到 {len(self.search_results)} 个匹配项 - 按 F3 查找下一个, Shift+F3 查找上一个, ESC 退出"
            return f"搜索模式 - 输入搜索词后按 Enter 开始搜索, ESC 退出"

        return "帮助: Ctrl-S = 保存文件 | Ctrl-Q = 退出编辑 | Ctrl-F = 查找 | F3 = 查找下一个"

    def start_search(self):
        """启动搜索模式"""
        self.search_mode = True
        self.search_results = []
        self.current_search_index = -1

        # 使用上一次的搜索词
        if self.last_search_query:
            self.search_input.text = self.last_search_query

        # 设置焦点到搜索输入框
        self.application.layout.focus(self.search_input)
        self.set_status_message("输入搜索词后按 Enter 开始搜索")

    def end_search(self):
        """结束搜索模式"""
        self.search_mode = False
        # 保存搜索词历史
        if self.search_query and self.search_query not in self.search_history:
            self.search_history.append(self.search_query)
        # 更新词法分析器，移除搜索高亮
        self.text_area.lexer = NanoLexer(self.syntax)
        # 设置焦点回主文本区域
        self.application.layout.focus(self.text_area)
        self.set_status_message("结束搜索模式")

    def perform_search(self, query=None):
        """执行搜索操作"""
        if query is None:
            query = self.search_input.text.strip()

        if not query:
            self.set_status_message("请输入搜索词")
            return False

        self.search_query = query
        self.last_search_query = query

        text = self.text_area.text
        self.search_results = []
        start = 0

        # 查找所有匹配项
        while True:
            pos = text.find(self.search_query, start)
            if pos == -1:
                break
            self.search_results.append(pos)
            start = pos + len(self.search_query)

        if self.search_results:
            self.current_search_index = -1
            self.set_status_message("找到 %d 个匹配项", len(self.search_results))

            # 更新词法分析器，添加搜索高亮
            self.text_area.lexer = NanoLexer(self.syntax, self.search_query)
            return True
        else:
            self.set_status_message("未找到匹配项: %s", self.search_query)
            return False

    def jump_to_search_result(self, index=None):
        """跳转到搜索结果"""
        if not self.search_results:
            return

        if index is None:
            index = self.current_search_index

        if index < 0 or index >= len(self.search_results):
            return

        self.current_search_index = index
        pos = self.search_results[self.current_search_index]

        # 设置光标位置
        self.text_area.buffer.cursor_position = pos

        # 滚动到可见区域
        self.text_area.buffer.cursor_position = pos
        self.text_area.buffer.cursor_position = pos + len(self.search_query)

        # 设置状态消息
        self.set_status_message("找到 %d 个匹配项, 当前第 %d 个",
                                len(self.search_results),
                                self.current_search_index + 1)

    def find_next(self):
        """查找下一个匹配项"""
        if not self.search_results:
            if not self.perform_search():
                return

        if self.current_search_index < 0:
            self.current_search_index = 0
        else:
            self.current_search_index = (self.current_search_index + 1) % len(self.search_results)

        self.jump_to_search_result()

    def find_previous(self):
        """查找上一个匹配项"""
        if not self.search_results:
            if not self.perform_search():
                return

        if self.current_search_index < 0:
            self.current_search_index = len(self.search_results) - 1
        else:
            self.current_search_index = (self.current_search_index - 1) % len(self.search_results)

        self.jump_to_search_result()

    def setup_key_bindings(self):
        """设置按键绑定"""
        kb = self.bindings

        @kb.add(CTRL_S)
        def _(event):
            self.save_file()

        @kb.add(CTRL_Q)
        def _(event):
            if self.dirty:
                self.set_status_message("警告!!! 文件存在未保存的修改。再按一次 Ctrl-Q 退出.")
                self.dirty = False
            else:
                event.app.exit()

        @kb.add(CTRL_F)
        def _(event):
            self.start_search()

        @kb.add('c-n')
        def _(event):
            self.find_next()

        @kb.add('c-p')
        def _(event):
            self.find_previous()

        @kb.add(ESC)
        def _(event):
            if self.search_mode:
                self.end_search()
            else:
                # 允许ESC退出应用
                event.app.exit()

        @kb.add('enter', filter=Condition(lambda: self.search_mode))
        def _(event):
            # 在搜索模式下按Enter，执行搜索
            if self.search_input.text.strip():
                self.perform_search()
                if self.search_results:
                    self.jump_to_search_result(0)
            else:
                self.set_status_message("请输入搜索词")

        # 文本变化时标记为已修改
        def on_text_changed(buf):
            self.dirty = True

        self.text_area.buffer.on_text_changed += on_text_changed

        # 搜索输入框变化时实时搜索
        def on_search_changed(buf):
            query = buf.text.strip()
            if query:  # 至少1个字符才实时搜索
                self.perform_search(query)
                if self.search_results:
                    self.jump_to_search_result(0)

        self.search_input.buffer.on_text_changed += on_search_changed

    def get_content(self):
        """获取当前编辑内容"""
        return self.text_area.text

    def run(self):
        """运行编辑器"""
        self.set_status_message("帮助: Ctrl-S = 保存文件 | Ctrl-Q = 退出编辑 | Ctrl-F = 查找 | F3 = 查找下一个")
        self.application.run()


def run_editor(file_name) -> str:
    editor = NanoEditor(file_name)
    editor.run()
    return editor.get_content()


def main():
    if len(sys.argv) != 2:
        print("Usage: nano <filename>")
        sys.exit(1)

    editor = NanoEditor(sys.argv[1])
    editor.run()


if __name__ == "__main__":
    main()


from rich.live import Live
from rich.panel import Panel
from rich.markdown import Markdown
from rich.text import Text

from autocoder_nano.core import AutoLLM
from autocoder_nano.actypes import AutoCoderArgs
from autocoder_nano.utils.printer_utils import Printer


printer = Printer


def stream_chat_display(
        chat_llm: AutoLLM, args: AutoCoderArgs, conversations: list[dict], max_history_lines: int = 15, max_height: int = 25
) -> str:
    v = chat_llm.stream_chat_ai(conversations=conversations, model=args.chat_model)

    lines_buffer = []
    assistant_response = ""
    current_line = ""

    try:
        with Live(Panel("", title="Response", style="cyan"), refresh_per_second=12) as live:
            for chunk in v:
                if chunk.choices and chunk.choices[0].delta.content:
                    content = chunk.choices[0].delta.content
                    assistant_response += content

                    # 处理换行符分割
                    parts = (current_line + content).split('\n')

                    # 最后一部分是未完成的新行
                    if len(parts) > 1:
                        # 将完整行加入缓冲区
                        lines_buffer.extend(parts[:-1])
                        # 保留最近N行历史
                        if len(lines_buffer) > max_history_lines:
                            del lines_buffer[0: len(lines_buffer) - max_history_lines]
                    # 更新当前行（最后未完成的部分）
                    current_line = parts[-1]
                    # 构建显示内容 = 历史行 + 当前行
                    display_content = '\n'.join(lines_buffer[-max_history_lines:] + [current_line])

                    live.update(
                        Panel(Markdown(display_content), title="模型返回", border_style="cyan",
                              height=min(max_height, live.console.height - 4))
                    )

            # 处理最后未换行的内容
            if current_line:
                lines_buffer.append(current_line)

            # 最终完整渲染
            live.update(
                Panel(Markdown(assistant_response), title="模型返回", border_style="dim blue")
            )
    except Exception as e:
        printer.print_panel(Text(f"{str(e)}", style="red"), title="模型返回", center=True)

    return assistant_response


__all__ = ["stream_chat_display"]



import os

from autocoder_nano.core import AutoLLM
from autocoder_nano.actypes import AutoCoderArgs
from autocoder_nano.rag import rag_build_cache, rag_retrieval


def rag_retrieval_test():
    doc_path = ""

    llm = AutoLLM()
    llm.setup_sub_client(
        client_name="ark-deepseek-v3",
        api_key="",
        base_url="https://ark.cn-beijing.volces.com/api/v3",
        model_name="deepseek-v3-250324"
    )
    llm.setup_sub_client(
        client_name="emb",
        api_key="",
        base_url="https://ark.cn-beijing.volces.com/api/v3",
        model_name="doubao-embedding-text-240715"
    )

    args = AutoCoderArgs()
    args.model_max_input_length = 100000
    args.chunk_model = "ark-deepseek-v3"
    args.qa_model = "ark-deepseek-v3"
    args.recall_model = "ark-deepseek-v3"
    args.emb_model = "emb"
    args.enable_hybrid_index = True
    args.query = ""

    print(rag_retrieval(llm=llm, args=args, path=doc_path))


def rag_build_cache_test():
    doc_path = ""

    llm = AutoLLM()
    llm.setup_sub_client(
        client_name="emb",
        api_key="",
        base_url="https://ark.cn-beijing.volces.com/api/v3",
        model_name="doubao-embedding-text-240715"
    )

    args = AutoCoderArgs()
    args.model_max_input_length = 100000
    args.chunk_model = "ark-deepseek-v3"
    args.qa_model = "ark-deepseek-v3"
    args.recall_model = "ark-deepseek-v3"
    args.emb_model = "emb"
    args.enable_hybrid_index = True
    args.required_exts = ".pdf,.doc"

    rag_build_cache(llm=llm, args=args, path=doc_path)


if __name__ == '__main__':
    rag_build_cache_test()


import json
import time
from concurrent.futures import ThreadPoolExecutor, as_completed
from typing import Callable, List, Dict

from autocoder_nano.core import AutoLLM
from autocoder_nano.core import prompt, extract_code
from autocoder_nano.actypes import SourceCode, AutoCoderArgs
from autocoder_nano.utils.printer_utils import Printer


printer = Printer()


class TokenLimiter:
    def __init__(
        self, count_tokens: Callable[[str], int], full_text_limit: int, segment_limit: int, buff_limit: int,
        llm: AutoLLM, args: AutoCoderArgs, disable_segment_reorder: bool,
    ):
        self.count_tokens = count_tokens
        self.full_text_limit = full_text_limit
        self.segment_limit = segment_limit
        self.buff_limit = buff_limit
        self.llm = llm
        self.args = args
        self.first_round_full_docs = []
        self.second_round_extracted_docs = []
        self.sencond_round_time = 0
        self.disable_segment_reorder = disable_segment_reorder

    @prompt()
    def extract_relevance_range_from_docs_with_conversation(
            self, conversations: List[Dict[str, str]], documents: List[str]
    ) -> str:
        """
        根据提供的文档和对话历史提取相关信息范围。

        输入:
        1. 文档内容:
        {% for doc in documents %}
        {{ doc }}
        {% endfor %}

        2. 对话历史:
        {% for msg in conversations %}
        <{{ msg.role }}>: {{ msg.content }}
        {% endfor %}

        任务:
        1. 分析最后一个用户问题及其上下文。
        2. 在文档中找出与问题相关的一个或多个重要信息段。
        3. 对每个相关信息段，确定其起始行号(start_line)和结束行号(end_line)。
        4. 信息段数量不超过4个。

        输出要求:
        1. 返回一个JSON数组，每个元素包含"start_line"和"end_line"。
        2. start_line和end_line必须是整数，表示文档中的行号。
        3. 行号从1开始计数。
        4. 如果没有相关信息，返回空数组[]。

        输出格式:
        严格的JSON数组，不包含其他文字或解释。

        示例:
        1.  文档：
            1 这是这篇动物科普文。
            2 大象是陆地上最大的动物之一。
            3 它们生活在非洲和亚洲。
            问题：大象生活在哪里？
            返回：[{"start_line": 2, "end_line": 3}]

        2.  文档：
            1 地球是太阳系第三行星，
            2 有海洋、沙漠，温度适宜，
            3 是已知唯一有生命的星球。
            4 太阳则是太阳系的唯一恒心。
            问题：地球的特点是什么？
            返回：[{"start_line": 1, "end_line": 3}]

        3.  文档：
            1 苹果富含维生素。
            2 香蕉含有大量钾元素。
            问题：橙子的特点是什么？
            返回：[]
        """

    def limit_tokens(
            self, relevant_docs: List[SourceCode], conversations: List[Dict[str, str]], index_filter_workers: int,
    ) -> List[SourceCode]:
        final_relevant_docs = []
        token_count = 0
        doc_num_count = 0

        reorder_relevant_docs = []

        # 文档分段（单个文档过大）和重排序逻辑
        # 1. 背景：在检索过程中，许多文档被切割成多个段落（segments）
        # 2. 问题：这些segments在召回时因为是按相关分做了排序可能是乱序的，不符合原文顺序，会强化大模型的幻觉。
        # 3. 目标：重新排序这些segments，确保来自同一文档的segments保持连续且按正确顺序排列。
        # 4. 实现方案：
        #    a) 方案一（保留位置）：统一文档的不同segments 根据chunk_index 来置换位置
        #    b) 方案二（当前实现）：遍历文档，发现某文档的segment A，立即查找该文档的所有其他segments，
        #       对它们进行排序，并将排序后多个segments插入到当前的segment A 位置中。
        # TODO:
        #     1. 未来根据参数决定是否开启重排以及重排的策略
        if not self.disable_segment_reorder:
            num_count = 0
            for doc in relevant_docs:
                num_count += 1
                reorder_relevant_docs.append(doc)
                if "original_doc" in doc.metadata and "chunk_index" in doc.metadata:
                    original_doc_name = doc.metadata["original_doc"]

                    temp_docs = []
                    for temp_doc in relevant_docs[num_count:]:
                        if "original_doc" in temp_doc.metadata and "chunk_index" in temp_doc.metadata:
                            if temp_doc.metadata["original_doc"] == original_doc_name:
                                if temp_doc not in reorder_relevant_docs:
                                    temp_docs.append(temp_doc)

                    temp_docs.sort(key=lambda x: x.metadata["chunk_index"])
                    reorder_relevant_docs.extend(temp_docs)
        else:
            reorder_relevant_docs = relevant_docs

        # 非窗口分区实现
        for doc in reorder_relevant_docs:
            doc_tokens = self.count_tokens(doc.source_code)
            doc_num_count += 1
            if token_count + doc_tokens <= self.full_text_limit + self.segment_limit:
                final_relevant_docs.append(doc)
                token_count += doc_tokens
            else:
                break

        # 如果窗口无法放下所有的相关文档，则需要分区
        if len(final_relevant_docs) < len(reorder_relevant_docs):
            printer.print_text(f"窗口无法放下所有的相关文档, 开始分区处理", style="yellow")
            # 先填充full_text分区
            token_count = 0
            new_token_limit = self.full_text_limit
            doc_num_count = 0
            for doc in reorder_relevant_docs:
                doc_tokens = self.count_tokens(doc.source_code)
                doc_num_count += 1
                if token_count + doc_tokens <= new_token_limit:
                    self.first_round_full_docs.append(doc)
                    token_count += doc_tokens
                else:
                    break

            if len(self.first_round_full_docs) > 0:
                remaining_tokens = (self.full_text_limit + self.segment_limit - token_count)
            else:
                printer.print_text("整个文本区域为空，这可能是由于单个文档过长导致的", style="yellow")
                remaining_tokens = self.full_text_limit + self.segment_limit

            # 继续填充segment分区
            sencond_round_start_time = time.time()
            remaining_docs = reorder_relevant_docs[len(self.first_round_full_docs):]
            printer.print_key_value(
                items={"首轮文档数": f"{len(self.first_round_full_docs)}", "剩余文档数": f"{len(remaining_docs)}"},
                title="第一轮 DocLimit"
            )

            self.llm.setup_default_model_name(self.args.chunk_model)
            with ThreadPoolExecutor(max_workers=index_filter_workers or 5) as executor:
                future_to_doc = {
                    executor.submit(self.process_range_doc, doc, conversations): doc
                    for doc in remaining_docs
                }

                for future in as_completed(future_to_doc):
                    doc = future_to_doc[future]
                    try:
                        result = future.result()
                        if result and remaining_tokens > 0:
                            self.second_round_extracted_docs.append(result)
                            tokens = result.tokens
                            if tokens > 0:
                                remaining_tokens -= tokens
                            else:
                                printer.print_text(f"文档 {doc.module_name} 的标记数量为 0 或负数", style="yellow")
                    except Exception as exc:
                        printer.print_text(f"处理文档 {doc.module_name} 时发生异常: {exc}", style="red")

            final_relevant_docs = (self.first_round_full_docs + self.second_round_extracted_docs)
            self.sencond_round_time = time.time() - sencond_round_start_time
            printer.print_key_value(
                items={
                    "第二轮文档数": f"{self.second_round_extracted_docs}",
                    "处理耗时": f"{self.sencond_round_time:.2f} 秒"},
                title="第二轮 DocLimit"
            )

        return final_relevant_docs

    def process_range_doc(
            self, doc: SourceCode, conversations: List[Dict[str, str]], max_retries=3
    ) -> SourceCode | None:
        for attempt in range(max_retries):
            content = ""
            try:
                source_code_with_line_number = ""
                source_code_lines = doc.source_code.split("\n")
                for idx, line in enumerate(source_code_lines):
                    source_code_with_line_number += f"{idx + 1} {line}\n"

                llm = self.llm

                extracted_info = (
                    self.extract_relevance_range_from_docs_with_conversation
                    .with_llm(llm)
                    .run(conversations, [source_code_with_line_number])
                )
                json_str = extract_code(extracted_info.output)[0][1]
                json_objs = json.loads(json_str)

                for json_obj in json_objs:
                    start_line = json_obj["start_line"] - 1
                    end_line = json_obj["end_line"]
                    chunk = "\n".join(source_code_lines[start_line:end_line])
                    content += chunk + "\n"

                return SourceCode(
                    module_name=doc.module_name,
                    source_code=content.strip(),
                    tokens=self.count_tokens(content),
                    metadata={
                        "original_doc": doc.module_name,
                        "chunk_ranges": json_objs,
                    },
                )
            except Exception as e:
                if attempt < max_retries - 1:
                    printer.print_text(f"处理文档 {doc.module_name} 时出错，正在重试(尝试次数:{attempt + 1})错误：{str(e)}",
                                       style="yellow")
                else:
                    printer.print_text(f"在 {max_retries} 次尝试后仍未能处理文档 {doc.module_name}：{str(e)}",
                                       style="red")
                    return SourceCode(
                        module_name=doc.module_name, source_code="", tokens=0
                    )


import typing
from typing import Union
from abc import ABC, abstractmethod

from autocoder_nano.agent.agentic_edit_types import *
from autocoder_nano.actypes import AutoCoderArgs

if typing.TYPE_CHECKING:
    from autocoder_nano.agent.agentic_runtime import AgenticRuntime
    from autocoder_nano.agent.agentic_sub import SubAgents


class BaseToolResolver(ABC):
    def __init__(
            self, agent: Optional[Union['AgenticRuntime', 'SubAgents']],
            tool: BaseTool, args: AutoCoderArgs
    ):
        """
        Initializes the resolver.
        Args:
            agent: The AutoCoder agent instance.
            tool: The Pydantic model instance representing the tool call.
            args: Additional arguments needed for execution (e.g., source_dir).
        """
        self.agent = agent
        self.tool = tool
        # self.args = args

    @abstractmethod
    def resolve(self) -> ToolResult:
        """
        Executes the tool's logic.
        Returns:
            A ToolResult object indicating success or failure and a message.
        """
        pass

    @abstractmethod
    def guide(self) -> str:
        pass


import os
import json
import uuid
from datetime import datetime


# New helper function for cleaning up logs
def _cleanup_logs(logs_dir: str, max_files: int = 100):
    """
    Cleans up old log files in the specified directory, keeping only the most recent ones.
    Log files are expected to follow the naming convention: <YYYYmmdd_HHMMSS>_<uuid>_<suffix>.md
    """
    # logger.debug(f"开始清理日志目录: {logs_dir}，最大保留文件数: {max_files}")
    if not os.path.isdir(logs_dir):
        # logger.debug(f"日志目录 {logs_dir} 不存在，无需清理。")
        return

    log_files = []
    for filename in os.listdir(logs_dir):
        if filename.endswith(".md"):
            parts = filename.split('_')
            # Expected format: <YYYYmmdd_HHMMSS>_<uuid>_<suffix>.md
            # parts[0] should be the full timestamp string "YYYYmmdd_HHMMSS"
            if len(parts) >= 2:  # At least timestamp and uuid part (suffix might be empty or complex)
                timestamp_str = parts[0]
                try:
                    # Validate the timestamp format
                    datetime.strptime(timestamp_str, "%Y%m%d_%H%M%S")
                    log_files.append((timestamp_str, os.path.join(logs_dir, filename)))
                except ValueError:
                    # logger.debug(f"文件名 {filename} 的时间戳部分 ({timestamp_str}) 格式不正确，跳过。")
                    continue
            else:
                pass
                # Log the parts for better debugging if needed
                # logger.debug(f"文件名 {filename} (分割后: {parts}) 不符合预期的下划线分割数量 (至少需要 <timestamp>_<uuid>_...)，跳过。")

    # Sort by timestamp (oldest first)
    log_files.sort(key=lambda x: x[0])

    if len(log_files) > max_files:
        files_to_delete_count = len(log_files) - max_files
        # logger.info(f"日志文件数量 ({len(log_files)}) 超过限制 ({max_files})，将删除 {files_to_delete_count} 个最旧的文件。")
        for i in range(files_to_delete_count):
            file_to_delete_timestamp, file_to_delete_path = log_files[i]
            try:
                os.remove(file_to_delete_path)
                # logger.info(f"已删除旧日志文件: {file_to_delete_path} (时间戳: {file_to_delete_timestamp})")
            except OSError as e:
                pass
                # logger.warning(f"删除日志文件 {file_to_delete_path} 失败: {str(e)}")
                # logger.exception(e)  # Log stack trace
    else:
        pass
        # logger.debug(f"日志文件数量 ({len(log_files)}) 未超过限制 ({max_files})，无需删除。")


def save_formatted_log(project_root, json_text, suffix):
    """
    Save a JSON log as a formatted markdown file under project_root/.auto-coder/logs/agentic.
    Filename: <YYYYmmdd_HHMMSS>_<uuid>_<suffix>.md
    Also cleans up old logs in the directory, keeping the latest 100.
    Args:
        project_root (str): The root directory of the project.
        json_text (str): The JSON string to be formatted and saved.
        suffix (str): The suffix for the filename.
    """
    # Prepare directory (logs_dir is needed for cleanup first)
    logs_dir = os.path.join(project_root, ".auto-coder", "logs", "agentic")

    # Cleanup old logs BEFORE saving the new one
    try:
        _cleanup_logs(logs_dir)  # Default to keep 100 files
    except Exception as e:
        pass
        # logger.warning(f"日志清理过程中发生错误: {str(e)}")
        # logger.exception(e)
        # Log cleanup failure should not prevent the main functionality

    # Parse JSON
    try:
        data = json.loads(json_text)
    except Exception as e:
        # logger.error(f"无效的 JSON 格式: {str(e)}") # Log error before raising
        # logger.exception(e) # Log stack trace
        raise ValueError(f"Invalid JSON provided: {e}")

    # Format as markdown with recursive depth
    def to_markdown(obj, level=1):
        lines = []
        if isinstance(obj, dict):
            for key, value in obj.items():
                lines.append(f"{'#' * (level + 1)} {key}\n")
                lines.extend(to_markdown(value, level + 1))
        elif isinstance(obj, list):
            for idx, item in enumerate(obj, 1):
                lines.append(f"{'#' * (level + 1)} Item {idx}\n")
                lines.extend(to_markdown(item, level + 1))
        else:
            lines.append(str(obj) + "\n")
        return lines

    md_lines = ["# Log Entry\n"]
    md_lines.extend(to_markdown(data, 1))
    md_content = "\n".join(md_lines)

    # Ensure directory exists
    # _cleanup_logs checks if dir exists but does not create it.
    # os.makedirs needs to be called to ensure the directory for the new log file.
    os.makedirs(logs_dir, exist_ok=True)

    # Prepare filename
    now = datetime.now().strftime("%Y%m%d_%H%M%S")
    unique_id = str(uuid.uuid4())
    filename = f"{now}_{unique_id}_{suffix}.md"
    filepath = os.path.join(logs_dir, filename)

    # Save file
    try:
        with open(filepath, "w", encoding="utf-8") as f:
            f.write(md_content)
        # logger.info(f"日志已保存至: {filepath}")
    except IOError as e:
        # logger.error(f"保存日志文件 {filepath} 失败: {str(e)}")
        # logger.exception(e) # Log stack trace
        raise  # Re-throw the exception so the caller knows saving failed

    return filepath


from typing import List, Generator, Any, Optional, Dict, Union

# from loguru import logger
from openai import OpenAI, Stream
from openai.types.chat import ChatCompletionChunk, ChatCompletion

from autocoder_nano.actypes import LLMRequest, LLMResponse, AutoCoderArgs, SingleOutputMeta
from autocoder_nano.utils.printer_utils import Printer, COLOR_INFO, COLOR_WARNING


printer = Printer()


class AutoLLM:
    def __init__(self):
        self.default_model_name = None
        self.sub_clients = {}

        # llm printer prefix
        self.lpp = f"* (waiting for llm) "

    def setup_sub_client(self, client_name: str, api_key: str, base_url: str, model_name=""):
        self.sub_clients[client_name] = {
            "client": OpenAI(api_key=api_key, base_url=base_url),
            "model_name": model_name
        }
        return self

    def remove_sub_client(self, client_name: str):
        if client_name in self.sub_clients:
            del self.sub_clients[client_name]

    def get_sub_client(self, client_name: str):
        return self.sub_clients.get(client_name, None)

    def setup_default_model_name(self, model_name: str):
        self.default_model_name = model_name

    def stream_chat_ai(self, conversations, model=None) -> Stream[ChatCompletionChunk]:
        if not model and not self.default_model_name:
            raise Exception("model name is required")

        if not model:
            model = self.default_model_name

        model_name = self.sub_clients[model]["model_name"]
        # printer.print_text(f"模型调用[{model}], 模型名称[{model_name}], 调用函数[stream_chat_ai]",
        #                    style=COLOR_INFO, prefix=self.lpp)
        request = LLMRequest(
            model=model_name,
            messages=conversations
        )
        res = self._query(model, request, stream=True)
        return res

    def stream_chat_ai_ex(
            self, conversations, model: Optional[str] = None, role_mapping=None, delta_mode: bool = False,
            is_reasoning: bool = False, llm_config: dict | None = None
    ):
        if llm_config is None:
            llm_config = {}
        if not model:
            model = self.default_model_name

        client: OpenAI = self.sub_clients[model]["client"]
        model_name = self.sub_clients[model]["model_name"]

        # printer.print_text(f"模型调用[{model}], 模型名称[{model_name}], 调用函数[stream_chat_ai_ex]",
        #                    style=COLOR_INFO, prefix=self.lpp)

        request = LLMRequest(
            model=model_name,
            messages=conversations,
            stream=True
        )

        # if is_reasoning:
        #     response = client.chat.completions.create(
        #         messages=request.messages,
        #         model=request.model,
        #         stream=request.stream,
        #         stream_options={"include_usage": True},
        #         extra_headers={
        #             "HTTP-Referer": "https://auto-coder.chat",
        #             "X-Title": "auto-coder-nano"
        #         },
        #         **llm_config
        #     )
        # else:
        response = client.chat.completions.create(
            messages=conversations,
            model=model_name,
            temperature=llm_config.get("temperature", request.temperature),
            max_tokens=llm_config.get("max_tokens", request.max_tokens),
            top_p=llm_config.get("top_p", request.top_p),
            stream=request.stream,
            stream_options={"include_usage": True},
            **llm_config
        )

        last_meta = None

        if delta_mode:
            for chunk in response:
                if hasattr(chunk, "usage") and chunk.usage:
                    input_tokens_count = chunk.usage.prompt_tokens
                    generated_tokens_count = chunk.usage.completion_tokens
                else:
                    input_tokens_count = 0
                    generated_tokens_count = 0

                if not chunk.choices:
                    if last_meta:
                        yield (
                            "",
                            SingleOutputMeta(
                                input_tokens_count=input_tokens_count,
                                generated_tokens_count=generated_tokens_count,
                                reasoning_content="",
                                finish_reason=last_meta.finish_reason,
                            ),
                        )
                    continue

                content = chunk.choices[0].delta.content or ""

                reasoning_text = ""
                if hasattr(chunk.choices[0].delta, "reasoning_content"):
                    reasoning_text = chunk.choices[0].delta.reasoning_content or ""

                last_meta = SingleOutputMeta(
                    input_tokens_count=input_tokens_count,
                    generated_tokens_count=generated_tokens_count,
                    reasoning_content=reasoning_text,
                    finish_reason=chunk.choices[0].finish_reason,
                )
                yield content, last_meta
        else:
            s = ""
            all_reasoning_text = ""
            for chunk in response:
                if hasattr(chunk, "usage") and chunk.usage:
                    input_tokens_count = chunk.usage.prompt_tokens
                    generated_tokens_count = chunk.usage.completion_tokens
                else:
                    input_tokens_count = 0
                    generated_tokens_count = 0

                if not chunk.choices:
                    if last_meta:
                        yield (
                            s,
                            SingleOutputMeta(
                                input_tokens_count=input_tokens_count,
                                generated_tokens_count=generated_tokens_count,
                                reasoning_content=all_reasoning_text,
                                finish_reason=last_meta.finish_reason,
                            ),
                        )
                    continue

                content = chunk.choices[0].delta.content or ""
                reasoning_text = ""
                if hasattr(chunk.choices[0].delta, "reasoning_content"):
                    reasoning_text = chunk.choices[0].delta.reasoning_content or ""

                s += content
                all_reasoning_text += reasoning_text
                yield (
                    s,
                    SingleOutputMeta(
                        input_tokens_count=input_tokens_count,
                        generated_tokens_count=generated_tokens_count,
                        reasoning_content=all_reasoning_text,
                        finish_reason=chunk.choices[0].finish_reason,
                    ),
                )

    def chat_ai(self, conversations, model=None) -> LLMResponse:
        # conversations = [{"role": "user", "content": prompt_str}]  deepseek-chat
        if not model and not self.default_model_name:
            raise Exception("model name is required")

        if not model:
            model = self.default_model_name

        if isinstance(conversations, str):
            conversations = [{"role": "user", "content": conversations}]

        model_name = self.sub_clients[model]["model_name"]
        # printer.print_text(f"模型调用[{model}], 模型名称[{model_name}], 调用函数[chat_ai]",
        #                    style=COLOR_INFO, prefix=self.lpp)
        request = LLMRequest(
            model=model_name,
            messages=conversations
        )

        res = self._query(model, request)
        return LLMResponse(
            output=res.choices[0].message.content,
            input="",
            metadata={
                "id": res.id,
                "model": res.model,
                "created": res.created
            }
        )

    def _query(self, model_name: str, request: LLMRequest, stream=False) -> ChatCompletion | Stream[ChatCompletionChunk]:
        """ 与 LLM 交互 """
        response = self.sub_clients[model_name]["client"].chat.completions.create(
            model=request.model,
            messages=request.messages,
            stream=stream,
            max_tokens=request.max_tokens,
            temperature=request.temperature,
            top_p=request.top_p,
            n=request.n,
            stop=request.stop,
            presence_penalty=request.presence_penalty,
            frequency_penalty=request.frequency_penalty,
        )
        return response

    def embedding(self, text: List, model=None):
        if not model and not self.default_model_name:
            raise Exception("model name is required")

        if not model:
            model = self.default_model_name

        model_name = self.sub_clients[model]["model_name"]

        res = self.sub_clients[model]["client"].embeddings.create(
            model=model_name,
            input=text,
            encoding_format="float"
        )
        return LLMResponse(
            output=res.data[0].embedding,
            input="",
            metadata={
                "id": res.id,
                "model": res.model,
                "created": res.created
            }
        )


def stream_chat_with_continue(
        llm: AutoLLM, conversations: List[dict], llm_config: dict, args: AutoCoderArgs
) -> Generator[Any, None, None]:
    """ 流式处理并继续生成内容，直到完成 """
    count = 0
    current_metadata = None
    metadatas = {}
    while True:
        # 使用流式接口获取生成内容
        stream_generator = llm.stream_chat_ai_ex(
            conversations=conversations,
            model=args.chat_model,
            delta_mode=True,
            llm_config={**llm_config}
        )

        for _content, _meta in stream_generator:
            if current_metadata is None:
                current_metadata = _meta
                metadatas[count] = _meta
            else:
                metadatas[count] = _meta
                current_metadata.finish_reason = _meta.finish_reason
                current_metadata.reasoning_content = _meta.reasoning_content

            # Yield 当前的 StreamChatWithContinueResult
            current_metadata.generated_tokens_count = sum([v.generated_tokens_count for _, v in metadatas.items()])
            current_metadata.input_tokens_count = sum([v.input_tokens_count for _, v in metadatas.items()])
            yield _content, current_metadata

        # 检查是否需要继续生成
        if current_metadata.finish_reason != "length" or count >= args.generate_max_rounds:
            if count >= args.generate_max_rounds:
                printer.print_text(f"LLM生成达到的最大次数, 当前次数:{count}, 最大次数: {args.generate_max_rounds}, "
                                   f"Tokens: {current_metadata.generated_tokens_count}", style=COLOR_WARNING)
            break
        count += 1


import contextlib
import json
import os
import threading
import time
from pathlib import Path
from typing import List, Dict, Any, Optional, Callable, Union, Generator

from autocoder_nano.context.exceptions import (
    ContextManagerError, ConversationNotFoundError, MessageNotFoundError, ConcurrencyError
)
from autocoder_nano.context.models import Conversation, ConversationMessage
from autocoder_nano.context.file_locker import FileLocker
from autocoder_nano.context.cache import MemoryCache, CacheManager
from autocoder_nano.context.search import TextSearcher, FilterManager
from autocoder_nano.context.storage import FileStorage, IndexManager
from autocoder_nano.utils.printer_utils import Printer


printer = Printer()


class ContextManagerConfig:
    """对话管理器配置类"""

    storage_path: str = "./.auto-coder/context"
    max_cache_size: int = 100
    cache_ttl: float = 300.0
    lock_timeout: float = 10.0
    backup_enabled: bool = True
    backup_interval: float = 3600.0
    max_backups: int = 10
    enable_compression: bool = False
    log_level: str = "INFO"

    def __post_init__(self):
        """配置验证"""
        self._validate()

    def _validate(self):
        """验证配置数据"""
        # 验证存储路径
        if not self.storage_path or not isinstance(self.storage_path, str):
            raise ValueError("存储路径不能为空")

        # 验证缓存大小
        if not isinstance(self.max_cache_size, int) or self.max_cache_size <= 0:
            raise ValueError("缓存大小必须是正整数")

        # 验证缓存TTL
        if not isinstance(self.cache_ttl, (int, float)) or self.cache_ttl <= 0:
            raise ValueError("缓存TTL必须是正数")

        # 验证锁超时
        if not isinstance(self.lock_timeout, (int, float)) or self.lock_timeout <= 0:
            raise ValueError("锁超时时间必须是正数")

        # 验证备份间隔
        if not isinstance(self.backup_interval, (int, float)) or self.backup_interval <= 0:
            raise ValueError("备份间隔必须是正数")

        # 验证最大备份数
        if not isinstance(self.max_backups, int) or self.max_backups <= 0:
            raise ValueError("最大备份数必须是正整数")

        # 验证日志级别
        valid_levels = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
        if self.log_level not in valid_levels:
            raise ValueError(f"无效的日志级别: {self.log_level}，有效级别: {valid_levels}")

    def to_dict(self) -> Dict[str, Any]:
        """转换为字典"""
        return {
            "storage_path": self.storage_path,
            "max_cache_size": self.max_cache_size,
            "cache_ttl": self.cache_ttl,
            "lock_timeout": self.lock_timeout,
            "backup_enabled": self.backup_enabled,
            "backup_interval": self.backup_interval,
            "max_backups": self.max_backups,
            "enable_compression": self.enable_compression,
            "log_level": self.log_level
        }

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ContextManagerConfig":
        """从字典创建配置"""
        # 创建默认配置
        config = cls()
        # 更新配置字段
        for key, value in data.items():
            if hasattr(config, key):
                setattr(config, key, value)
        # 重新验证
        config._validate()
        return config

    def save_to_file(self, file_path: str):
        """保存配置到文件"""
        # 确保目录存在
        os.makedirs(os.path.dirname(file_path), exist_ok=True)
        with open(file_path, 'w', encoding='utf-8') as f:
            json.dump(self.to_dict(), f, indent=2, ensure_ascii=False)

    @classmethod
    def load_from_file(cls, file_path: str) -> "ContextManagerConfig":
        """从文件加载配置"""
        if not os.path.exists(file_path):
            raise FileNotFoundError(f"配置文件不存在: {file_path}")
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                data = json.load(f)
            return cls.from_dict(data)
        except json.JSONDecodeError as e:
            raise ValueError(f"配置文件格式错误: {e}")

    def copy(self) -> "ContextManagerConfig":
        """创建配置的深拷贝"""
        return ContextManagerConfig.from_dict(self.to_dict())

    def update(self, **kwargs):
        """更新配置字段"""
        # 先备份当前配置
        backup_values = {}
        for key in kwargs.keys():
            if hasattr(self, key):
                backup_values[key] = getattr(self, key)
            else:
                raise AttributeError(f"配置类没有属性: {key}")

        # 尝试更新
        try:
            for key, value in kwargs.items():
                setattr(self, key, value)
            # 重新验证
            self._validate()
        except Exception:
            # 如果验证失败，恢复原值
            for key, value in backup_values.items():
                setattr(self, key, value)
            raise

    def __repr__(self) -> str:
        """字符串表示"""
        return (f"ConversationManagerConfig("
                f"storage_path='{self.storage_path}', "
                f"max_cache_size={self.max_cache_size}, "
                f"cache_ttl={self.cache_ttl}, "
                f"lock_timeout={self.lock_timeout}, "
                f"backup_enabled={self.backup_enabled}, "
                f"log_level='{self.log_level}')")

    def __eq__(self, other) -> bool:
        """相等性比较"""
        if not isinstance(other, ContextManagerConfig):
            return False
        return self.to_dict() == other.to_dict()


class ContextManager:
    def __init__(self, config: Optional[ContextManagerConfig] = None):
        self.config = config or ContextManagerConfig()

        # 初始化组件
        self._init_storage()
        self._init_cache()
        self._init_search()
        self._init_locks()

        # 统计信息跟踪
        self._stats = {
            'conversations_created': 0,
            'conversations_loaded': 0,
            'messages_added': 0,
            'cache_hits': 0,
            'cache_misses': 0
        }

    def _init_storage(self):
        """初始化存储组件"""
        # 确保存储目录存在
        storage_path = Path(self.config.storage_path)
        storage_path.mkdir(parents=True, exist_ok=True)

        # 初始化存储后端
        self.storage = FileStorage(str(storage_path / "conversations"))

        # 初始化索引管理器
        self.index_manager = IndexManager(str(storage_path / "index"))

    def _init_cache(self):
        """初始化缓存系统"""
        # 为对话和消息使用基于字典的简单缓存
        self.conversation_cache = MemoryCache(
            max_size=self.config.max_cache_size,
            default_ttl=self.config.cache_ttl
        )
        self.message_cache = MemoryCache(
            max_size=self.config.max_cache_size * 10,  # 消息数量通常比对话多
            default_ttl=self.config.cache_ttl
        )

    def _init_search(self):
        """Initialize search and filtering systems."""
        self.text_searcher = TextSearcher()
        self.filter_manager = FilterManager()

    def _init_locks(self):
        """初始化 locks"""
        lock_dir = Path(self.config.storage_path) / "locks"
        lock_dir.mkdir(parents=True, exist_ok=True)
        self._lock_dir = str(lock_dir)

    def _get_conversation_lock_file(self, conversation_id: str) -> str:
        """获取对话的锁文件路径"""
        return os.path.join(self._lock_dir, f"{conversation_id}.lock")

    @contextlib.contextmanager
    def _conversation_lock(self, conversation_id: str, exclusive: bool = True) -> Generator[None, None, None]:
        """
        为对话操作获取锁
        Args:
            conversation_id: 需要加锁的对话ID
            exclusive: 是否获取独占（写）锁
        """
        lock_file = self._get_conversation_lock_file(conversation_id)
        locker = FileLocker(lock_file, timeout=self.config.lock_timeout)
        try:
            if exclusive:
                with locker.acquire_write_lock():
                    yield
            else:
                with locker.acquire_read_lock():
                    yield
        except (ConversationNotFoundError, MessageNotFoundError):
            # Re-raise these exceptions as-is (don't wrap in ConcurrencyError)
            raise
        except Exception as e:
            raise ConcurrencyError(f"Failed to acquire lock for conversation {conversation_id}: {e}")

    def create_conversation(
            self, name: str, description: Optional[str] = None, initial_messages: Optional[List[Dict[str, Any]]] = None,
            metadata: Optional[Dict[str, Any]] = None) -> str:
        """
        创建新对话
        Args:
            name: 对话名称
            description: 可选描述信息
            initial_messages: 初始消息列表（可选）
            metadata: 元数据字典（可选）
        Returns:
            所创建对话的ID
        Raises:
            ConversationManagerError: 当对话创建失败时抛出此异常
        """
        try:
            conversation = Conversation(name=name, description=description, metadata=metadata or {})
            if initial_messages:
                for msg_data in initial_messages:
                    message = ConversationMessage(
                        role=msg_data['role'],
                        content=msg_data['content'],
                        metadata=msg_data.get('metadata', {})
                    )
                    conversation.add_message(message)
            # Save conversation with locking
            with self._conversation_lock(conversation.conversation_id):
                # Save to storage
                self.storage.save_conversation(conversation.to_dict())
                # Update index
                self.index_manager.add_conversation(conversation.to_dict())
                # Cache the conversation
                self.conversation_cache.set(conversation.conversation_id, conversation.to_dict())
            # Update statistics
            self._stats['conversations_created'] += 1
            return conversation.conversation_id
        except Exception as e:
            raise ContextManagerError(f"创建会话失败: {e}")

    def get_conversation(self, conversation_id: str) -> Optional[Dict[str, Any]]:
        """
        通过ID获取对话
        Args:
            conversation_id: 对话的ID
        Returns:
            对话数据，如果未找到则返回None
        """
        try:
            # Try cache first
            cached_conversation = self.conversation_cache.get(conversation_id)
            if cached_conversation:
                self._stats['cache_hits'] += 1
                return cached_conversation

            self._stats['cache_misses'] += 1

            # Load from storage with read lock
            with self._conversation_lock(conversation_id, exclusive=False):
                conversation_data = self.storage.load_conversation(conversation_id)

                if conversation_data:
                    # Cache the loaded conversation
                    self.conversation_cache.set(conversation_id, conversation_data)
                    self._stats['conversations_loaded'] += 1
                    return conversation_data

                return None

        except Exception as e:
            raise ContextManagerError(f"Failed to get conversation {conversation_id}: {e}")

    def update_conversation(self, conversation_id: str, name: Optional[str] = None, description: Optional[str] = None,
                            metadata: Optional[Dict[str, Any]] = None) -> bool:
        """
        Update conversation metadata.
        Args:
            conversation_id: ID of the conversation to update
            name: New name (optional)
            description: New description (optional)
            metadata: New metadata (optional)
        Returns:
            True if update was successful
        Raises:
            ConversationNotFoundError: If conversation doesn't exist
        """
        try:
            with self._conversation_lock(conversation_id):
                # Load current conversation
                conversation_data = self.storage.load_conversation(conversation_id)
                if not conversation_data:
                    raise ConversationNotFoundError(conversation_id)

                # Create conversation object and update fields
                conversation = Conversation.from_dict(conversation_data)

                if name is not None:
                    conversation.name = name
                if description is not None:
                    conversation.description = description
                if metadata is not None:
                    conversation.metadata.update(metadata)

                conversation.updated_at = time.time()

                # Save updated conversation
                updated_data = conversation.to_dict()
                self.storage.save_conversation(updated_data)
                # Update index
                self.index_manager.update_conversation(updated_data)
                # Update cache
                self.conversation_cache.set(conversation_id, updated_data)

                return True

        except ConversationNotFoundError:
            raise
        except Exception as e:
            raise ContextManagerError(f"Failed to update conversation {conversation_id}: {e}")

    def delete_conversation(self, conversation_id: str) -> bool:
        """
        Delete a conversation.
        Args:
            conversation_id: ID of the conversation to delete
        Returns:
            True if deletion was successful
        """
        try:
            with self._conversation_lock(conversation_id):
                # Check if conversation exists
                if not self.storage.conversation_exists(conversation_id):
                    return False
                # Delete from storage
                self.storage.delete_conversation(conversation_id)
                # Remove from index
                self.index_manager.remove_conversation(conversation_id)
                # Remove from cache
                self.conversation_cache.delete(conversation_id)
                return True

        except Exception as e:
            raise ContextManagerError(f"删除会话 {conversation_id} 失败: {e}")

    def search_conversations(
            self, query: str, search_in_messages: bool = True, filters: Optional[Dict[str, Any]] = None,
            max_results: Optional[int] = None, min_score: float = 0.0) -> List[Dict[str, Any]]:
        """
        Search conversations.
        Args:
            query: Search query
            search_in_messages: Whether to search in message content
            filters: Optional filter criteria
            max_results: Maximum number of results
            min_score: Minimum relevance score
        Returns:
            List of matching conversations with scores
        """
        try:
            # Get all conversations
            conversations = self.index_manager.list_conversations()

            # Apply filters first if provided
            if filters:
                conversations = self.filter_manager.apply_filters(conversations, filters)

            # If search_in_messages is True, load full conversation data
            if search_in_messages:
                full_conversations = []
                for conv in conversations:
                    conv_id = conv.get('conversation_id')
                    if conv_id:
                        full_conv = self.get_conversation(conv_id)
                        if full_conv:
                            full_conversations.append(full_conv)
                conversations = full_conversations

            # Perform text search
            results = self.text_searcher.search_conversations(
                query, conversations, max_results, min_score
            )

            return [{'conversation': conv, 'score': score} for conv, score in results]

        except Exception as e:
            raise ContextManagerError(f"Failed to search conversations: {e}")

    def search_messages(
            self, conversation_id: str, query: str, filters: Optional[Dict[str, Any]] = None,
            max_results: Optional[int] = None, min_score: float = 0.0) -> List[Dict[str, Any]]:
        """
        Search messages in a conversation.
        Args:
            conversation_id: ID of the conversation
            query: Search query
            filters: Optional filter criteria
            max_results: Maximum number of results
            min_score: Minimum relevance score
        Returns:
            List of matching messages with scores
        filters 示例:
        filters = {
            "filters": [
                {
                    "field": "role",
                    "operator": "in",
                    "value": ["assistant"]  # 角色在指定列表中
                },
                {
                    "field": "timestamp",
                    "operator": "gte",  # 大于等于
                    "value": 1717000000  # 时间戳（Unix时间，秒级）
                }
            ],
            "operator": "and"  # 逻辑运算符："and"（与）或 "or"（或）
        }
        支持的运算符（operator）:
        eq 等于(适用所有类型)
        ne 不等于(适用所有类型)
        gt/gte 大于/大于等于(适用数字、时间戳)
        lt/lte 小于/小于等于(数字、时间戳)
        contains 包含子串(字符串)
        in 在列表中(所有类型)
        regex 正则匹配(字符串)
        """
        try:
            # Get conversation messages
            messages = self.get_messages(conversation_id)

            # Apply filters if provided
            if filters:
                messages = self.filter_manager.apply_filters(messages, filters)

            # Perform text search
            results = self.text_searcher.search_messages(
                query, messages, max_results, min_score
            )

            return [{'message': msg, 'score': score} for msg, score in results]

        except Exception as e:
            raise ContextManagerError(f"Failed to search messages: {e}")

    def list_conversations(self, limit: Optional[int] = None, offset: int = 0, filters: Optional[Dict[str, Any]] = None,
                           sort_by: str = 'updated_at', sort_desc: bool = True) -> List[Dict[str, Any]]:
        """
        List conversations with optional filtering and sorting.
        Args:
            limit: Maximum number of conversations to return
            offset: Number of conversations to skip
            filters: Optional filter criteria
            sort_by: Field to sort by
            sort_desc: Whether to sort in descending order
        Returns:
            List of conversation data
        """
        try:
            # Convert sort_desc boolean to sort_order string
            sort_order = 'desc' if sort_desc else 'asc'
            # Get conversations from index with sorting and pagination
            conversations = self.index_manager.list_conversations(
                limit=limit,
                offset=offset,
                sort_by=sort_by,
                sort_order=sort_order
            )
            # Apply filters if provided
            if filters:
                conversations = self.filter_manager.apply_filters(conversations, filters)

            return conversations

        except Exception as e:
            raise ContextManagerError(f"Failed to list conversations: {e}")

    def append_message(self, conversation_id: str, role: str, content: Union[str, Dict[str, Any], List[Any]],
                       metadata: Optional[Dict[str, Any]] = None) -> str:
        """
        Append a message to a conversation.
        Args:
            conversation_id: ID of the conversation
            role: Role of the message sender
            content: Message content
            metadata: Optional message metadata
        Returns:
            ID of the added message
        Raises:
            ConversationNotFoundError: If conversation doesn't exist
        """
        try:
            # Create message object
            message = ConversationMessage(
                role=role,
                content=content,
                metadata=metadata or {}
            )

            with self._conversation_lock(conversation_id):
                # Load conversation
                conversation_data = self.storage.load_conversation(conversation_id)
                if not conversation_data:
                    raise ConversationNotFoundError(conversation_id)

                # Add message to conversation
                conversation = Conversation.from_dict(conversation_data)
                conversation.add_message(message)

                # Save updated conversation
                updated_data = conversation.to_dict()
                self.storage.save_conversation(updated_data)

                # Update index
                self.index_manager.update_conversation(updated_data)

                # Update cache
                self.conversation_cache.set(conversation_id, updated_data)
                self.message_cache.set(f"{conversation_id}:{message.message_id}", message.to_dict())

            # Update statistics
            self._stats['messages_added'] += 1

            return message.message_id

        except ConversationNotFoundError:
            raise
        except Exception as e:
            raise ContextManagerError(f"Failed to append message to conversation {conversation_id}: {e}")

    def append_messages(self, conversation_id: str, messages: List[Dict[str, Any]]) -> List[str]:
        """
        Append multiple messages to a conversation.
        Args:
            conversation_id: ID of the conversation
            messages: List of message data dictionaries
        Returns:
            List of message IDs
        """
        message_ids = []
        for msg_data in messages:
            message_id = self.append_message(
                conversation_id,
                msg_data['role'],
                msg_data['content'],
                msg_data.get('metadata')
            )
            message_ids.append(message_id)
        return message_ids

    def get_messages(self, conversation_id: str, limit: Optional[int] = None, offset: int = 0,
                     message_ids: Optional[List[str]] = None) -> List[Dict[str, Any]]:
        """
        Get messages from a conversation.
        Args:
            conversation_id: ID of the conversation
            limit: Maximum number of messages to return
            offset: Number of messages to skip
            message_ids: Optional list of specific message IDs
        Returns:
            List of message data
        """
        try:
            # Get conversation
            conversation_data = self.get_conversation(conversation_id)
            if not conversation_data:
                raise ConversationNotFoundError(conversation_id)
            messages = conversation_data.get('messages', [])
            # Filter by message IDs if provided
            if message_ids:
                id_set = set(message_ids)
                messages = [msg for msg in messages if msg.get('message_id') in id_set]
            # Apply pagination
            end_idx = offset + limit if limit else None
            return messages[offset:end_idx]

        except ConversationNotFoundError:
            raise
        except Exception as e:
            raise ContextManagerError(f"Failed to get messages from conversation {conversation_id}: {e}")

    def get_message(self, conversation_id: str, message_id: str) -> Optional[Dict[str, Any]]:
        """
        Get a specific message.
        Args:
            conversation_id: ID of the conversation
            message_id: ID of the message
        Returns:
            Message data or None if not found
        """
        try:
            # Try cache first
            cached_message = self.message_cache.get(f"{conversation_id}:{message_id}")
            if cached_message:
                return cached_message
            # Get from conversation
            conversation_data = self.get_conversation(conversation_id)
            if not conversation_data:
                return None
            # Find message
            for message in conversation_data.get('messages', []):
                if message.get('message_id') == message_id:
                    # Cache the message
                    self.message_cache.set(f"{conversation_id}:{message_id}", message)
                    return message
            return None
        except Exception as e:
            raise ContextManagerError(f"Failed to get message {message_id}: {e}")

    def update_message(self, conversation_id: str, message_id: str,
                       content: Optional[Union[str, Dict[str, Any], List[Any]]] = None,
                       metadata: Optional[Dict[str, Any]] = None) -> bool:
        """
        Update a message.
        Args:
            conversation_id: ID of the conversation
            message_id: ID of the message to update
            content: New content (optional)
            metadata: New metadata (optional)
        Returns:
            True if update was successful
        """
        try:
            with self._conversation_lock(conversation_id):
                # Load conversation
                conversation_data = self.storage.load_conversation(conversation_id)
                if not conversation_data:
                    raise ConversationNotFoundError(conversation_id)
                conversation = Conversation.from_dict(conversation_data)

                # Find and update message
                for i, message_data in enumerate(conversation.messages):
                    msg = ConversationMessage.from_dict(message_data)
                    if msg.message_id == message_id:
                        # Update message fields
                        if content is not None:
                            msg.content = content
                        if metadata is not None:
                            msg.metadata.update(metadata)

                        # Update timestamp
                        msg.timestamp = time.time()

                        # Replace in conversation
                        conversation.messages[i] = msg.to_dict()
                        conversation.updated_at = time.time()

                        # Save updated conversation
                        updated_data = conversation.to_dict()
                        self.storage.save_conversation(updated_data)

                        # Update index
                        self.index_manager.update_conversation(updated_data)

                        # Update caches
                        self.conversation_cache.set(conversation_id, updated_data)
                        self.message_cache.set(f"{conversation_id}:{message_id}", msg.to_dict())

                        return True

                raise MessageNotFoundError(message_id)

        except (ConversationNotFoundError, MessageNotFoundError):
            raise
        except Exception as e:
            raise ContextManagerError(f"更新会话消息失败 {message_id}: {e}")

    def delete_message(self, conversation_id: str, message_id: str) -> bool:
        """
        Delete a message from a conversation.
        Args:
            conversation_id: ID of the conversation
            message_id: ID of the message to delete
        Returns:
            True if deletion was successful
        """
        try:
            with self._conversation_lock(conversation_id):
                # Load conversation
                conversation_data = self.storage.load_conversation(conversation_id)
                if not conversation_data:
                    raise ConversationNotFoundError(conversation_id)

                conversation = Conversation.from_dict(conversation_data)

                # Find and remove message
                original_count = len(conversation.messages)
                conversation.messages = [
                    msg for msg in conversation.messages
                    if msg.get('message_id') != message_id
                ]

                if len(conversation.messages) == original_count:
                    raise MessageNotFoundError(message_id)

                conversation.updated_at = time.time()

                # Save updated conversation
                updated_data = conversation.to_dict()
                self.storage.save_conversation(updated_data)

                # Update index
                self.index_manager.update_conversation(updated_data)

                # Update caches
                self.conversation_cache.set(conversation_id, updated_data)
                self.message_cache.delete(f"{conversation_id}:{message_id}")

                return True

        except (ConversationNotFoundError, MessageNotFoundError):
            raise
        except Exception as e:
            raise ContextManagerError(f"从会话中删除消息失败 {message_id}: {e}")

    def get_statistics(self) -> Dict[str, Any]:
        """
        Get manager statistics.
        Returns:
            Dictionary with statistics
        """
        cache_stats = {
            'conversation_cache_size': self.conversation_cache.size(),
            'message_cache_size': self.message_cache.size()
        }

        return {
            **self._stats,
            'cache_stats': cache_stats,
            'total_conversations': len(self.index_manager.list_conversations()),
            'current_conversation_id': self.get_current_conversation_id(),
            'storage_path': self.config.storage_path
        }

    def health_check(self) -> Dict[str, Any]:
        """
        Perform health check of all components.
        Returns:
            Health status dictionary
        """
        health_status = {
            'status': 'healthy',
            'storage': True,
            'cache': True,
            'index': True,
            'search': True,
            'issues': []
        }

        try:
            # Check storage
            if not os.path.exists(self.config.storage_path):
                health_status['storage'] = False
                health_status['issues'].append('Storage directory not accessible')

            # Check cache
            conv_cache_size = self.conversation_cache.size()
            if conv_cache_size > self.config.max_cache_size:
                health_status['issues'].append('Conversation cache size exceeds limit')

            # Check index consistency
            try:
                conversations = self.index_manager.list_conversations()
                health_status['total_conversations'] = len(conversations)
            except Exception as e:
                health_status['index'] = False
                health_status['issues'].append(f'Index error: {e}')

            # Determine overall status
            if not all([health_status['storage'], health_status['cache'],
                        health_status['index'], health_status['search']]):
                health_status['status'] = 'degraded'

            if health_status['issues']:
                health_status['status'] = 'warning' if health_status['status'] == 'healthy' else health_status['status']

        except Exception as e:
            health_status['status'] = 'unhealthy'
            health_status['issues'].append(f'Health check failed: {e}')

        return health_status

    @contextlib.contextmanager
    def transaction(self, conversation_id: str) -> Generator[None, None, None]:
        """
        Transaction context manager for atomic operations.
        Args:
            conversation_id: ID of the conversation for the transaction
        """
        with self._conversation_lock(conversation_id):
            try:
                yield
            except Exception:
                # In a full implementation, we would rollback changes here
                # For now, we just re-raise the exception
                raise

    def clear_cache(self):
        """Clear all caches."""
        self.conversation_cache.clear()
        self.message_cache.clear()

    def rebuild_index(self):
        """Rebuild the conversation index from storage."""
        try:
            # Clear existing index
            # self.index_manager._index.clear()

            # Load all conversations from storage and rebuild index
            conversation_ids = self.storage.list_conversations()

            for conv_id in conversation_ids:
                conversation_data = self.storage.load_conversation(conv_id)
                if conversation_data:
                    self.index_manager.add_conversation(conversation_data)

        except Exception as e:
            raise ContextManagerError(f"Failed to rebuild index: {e}")

    def close(self):
        """Clean up resources."""
        # Clear caches
        self.clear_cache()

        # Save any pending index changes
        try:
            self.index_manager._save_index()
        except Exception:
            pass  # Ignore errors during cleanup

    def set_current_conversation(self, conversation_id: str) -> bool:
        """
        设置当前对话。
        Args:
            conversation_id: 要设置为当前对话的ID
        Returns:
            True if setting was successful
        Raises:
            ConversationNotFoundError: 如果对话不存在
        """
        try:
            # 验证对话是否存在
            conversation_data = self.get_conversation(conversation_id)
            if not conversation_data:
                raise ConversationNotFoundError(conversation_id)

            # 设置当前对话
            success = self.index_manager.set_current_conversation(conversation_id)
            if not success:
                raise ContextManagerError(f"Failed to set current conversation: {conversation_id}")

            return True

        except ConversationNotFoundError:
            raise
        except Exception as e:
            raise ContextManagerError(f"Failed to set current conversation {conversation_id}: {e}")

    def get_current_conversation_id(self) -> Optional[str]:
        """
        获取当前对话ID。
        Returns:
            当前对话ID，如果未设置返回None
        """
        try:
            return self.index_manager.get_current_conversation_id()
        except Exception as e:
            raise ContextManagerError(f"Failed to get current conversation ID: {e}")

    def get_current_conversation(self) -> Optional[Dict[str, Any]]:
        """
        获取当前对话的完整数据。
        Returns:
            当前对话的数据字典，如果未设置或对话不存在返回None
        """
        try:
            current_id = self.get_current_conversation_id()
            if not current_id:
                return None
            return self.get_conversation(current_id)

        except Exception as e:
            raise ContextManagerError(f"Failed to get current conversation: {e}")

    def clear_current_conversation(self) -> bool:
        """
        清除当前对话设置。
        Returns:
            True if clearing was successful
        """
        try:
            success = self.index_manager.clear_current_conversation()
            if not success:
                raise ContextManagerError("Failed to clear current conversation")
            return True
        except Exception as e:
            raise ContextManagerError(f"Failed to clear current conversation: {e}")

    def append_message_to_current(
            self, role: str, content: Union[str, Dict[str, Any], List[Any]], metadata: Optional[Dict[str, Any]] = None
    ) -> str:
        """
        向当前对话添加消息。
        Args:
            role: 消息角色
            content: 消息内容
            metadata: 可选的消息元数据
        Returns:
            消息ID
        Raises:
            ConversationManagerError: 如果没有设置当前对话或添加失败
        """
        try:
            current_id = self.get_current_conversation_id()
            if not current_id:
                raise ContextManagerError("未设置当前会话")

            return self.append_message(current_id, role, content, metadata)

        except Exception as e:
            raise ContextManagerError(f"向当前会话新增消息失败: {e}")


class ContextManagerSingleton:
    """对话管理器的单例类，确保全局只有一个实例"""
    _instance: Optional[ContextManager] = None
    _lock = threading.Lock()
    _config: Optional[ContextManagerConfig] = None

    @classmethod
    def get_instance(cls, config: Optional[ContextManagerConfig] = None) -> ContextManager:
        """
        获取对话管理器实例
        Args:
            config: 配置对象，如果为None则使用默认配置
        Returns:
            PersistConversationManager实例
        """
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    if config is None:
                        config = cls._get_default_config()
                    cls._config = config
                    cls._instance = ContextManager(config)
        return cls._instance

    @classmethod
    def reset_instance(cls, config: Optional[ContextManagerConfig] = None):
        """
        重置实例，用于测试或配置更改时
        Args:
            config: 新的配置对象
        """
        with cls._lock:
            cls._instance = None
            cls._config = None
            if config is not None:
                cls._instance = ContextManager(config)
                cls._config = config

    @classmethod
    def _get_default_config(cls) -> ContextManagerConfig:
        """获取默认配置"""
        # 默认存储路径为当前工作目录下的 .auto-coder/context
        default_storage_path = os.path.join(os.getcwd(), ".auto-coder", "context")
        return ContextManagerConfig()

    @classmethod
    def get_config(cls) -> Optional[ContextManagerConfig]:
        """获取当前使用的配置"""
        return cls._config


def get_context_manager(config: Optional[ContextManagerConfig] = None) -> ContextManager:
    """
    获取全局对话管理器实例
    这是一个便捷函数，内部使用单例模式确保全局只有一个实例。
    首次调用时会创建实例，后续调用会返回同一个实例。
    Args:
        config: 可选的配置对象。如果为None，将使用默认配置。
               注意：只有在首次调用时，config参数才会生效。
    Returns:
        PersistConversationManager: 对话管理器实例
    Example:
        ```python
        # 使用默认配置
        manager = get_conversation_manager()

        # 使用自定义配置（仅在首次调用时生效）
        config = ConversationManagerConfig(
            storage_path="./my_conversations",
            max_cache_size=200
        )
        manager = get_conversation_manager(config)

        # 创建对话
        conv_id = manager.create_conversation(
            name="测试对话",
            description="这是一个测试对话"
        )
        ```
    """
    return ContextManagerSingleton.get_instance(config)


def reset_context_manager(config: Optional[ContextManagerConfig] = None):
    """
    重置全局对话管理器实例
    用于测试或需要更改配置时重置实例。
    Args:
        config: 新的配置对象，如果为None则在下次调用get_conversation_manager时使用默认配置
    Example:
        ```python
        # 重置为默认配置
        reset_conversation_manager()
        # 重置为新配置
        new_config = ConversationManagerConfig(storage_path="./new_path")
        reset_conversation_manager(new_config)
        ```
    """
    ContextManagerSingleton.reset_instance(config)


def get_context_manager_config() -> Optional[ContextManagerConfig]:
    """
    获取当前对话管理器使用的配置
    Returns:
        当前配置对象，如果还未初始化则返回None
    """
    return ContextManagerSingleton.get_config()


import typing
from typing import Optional, Union

from autocoder_nano.agent.agentic_edit_tools.base_tool_resolver import BaseToolResolver
from autocoder_nano.agent.agentic_edit_types import CallSkillsTool, ToolResult
from autocoder_nano.actypes import AutoCoderArgs
from autocoder_nano.utils.printer_utils import Printer

if typing.TYPE_CHECKING:
    from autocoder_nano.agent.agentic_runtime import AgenticRuntime


printer = Printer()


class CallSkillsToolResolver(BaseToolResolver):
    def __init__(self, agent: Optional[Union['AgenticRuntime', 'SkillAgent']], tool: CallSkillsTool,
                 args: AutoCoderArgs):
        super().__init__(agent, tool, args)
        self.tool: CallSkillsTool = tool

    def resolve(self) -> ToolResult:
        try:
            from autocoder_nano.agent.agentic_skills import SkillRegistry, SkillLoader, SkillAgent, SkillLevel
            skill_name = self.tool.skill_name
            request = self.tool.request

            _registry = SkillRegistry(args=self.agent.args)
            _registry.scan_skills()
            _skill_path = _registry.get_skill_path(skill_name)
            _loader = SkillLoader()
            _level = SkillLevel.RESOURCES
            _skill_content = _loader.load_skill(_skill_path, _level)

            # 初始化及添加技能概述
            skill_prompt = [
                f"# Skill: {_skill_content.metadata.name}",
                f"\n"
                f"**Description**: {_skill_content.metadata.description}"
                f"\n"
            ]
            # 添加技能主体
            if _skill_content.body:
                skill_prompt.append(_skill_content.body)
                skill_prompt.append("\n")
            # 添加 script 列表
            if _skill_content.scripts:
                skill_prompt.append("## Script Path")
                skill_prompt.append("\n")
                for script_name, script_content in _skill_content.scripts.items():
                    skill_prompt.append(f"- {script_name}")
                skill_prompt.append("\n")
            # 添加参考文档（如果有）
            if _skill_content.references:
                skill_prompt.append("## References")
                for ref_name, ref_content in _skill_content.references.items():
                    skill_prompt.append(f"\n### {ref_name}")
                    skill_prompt.append(ref_content)
                skill_prompt.append("\n")

            skills_agent_define = {
                "skills": {
                    "type": "sub",
                    "description": "",
                    "call": "",
                    "prompt": skill_prompt,
                    "tools": [
                        "search_files",
                        "list_files",
                        "read_file",
                        "execute_command",
                        "write_to_file",
                        "attempt_completion",
                    ]
                }
            }

            skill_agent = SkillAgent(
                self.agent.args,
                self.agent.llm,
                skills_agent_define)
            completion_status, completion_text = skill_agent.run_skills_agent(skill_name, request)

            if completion_status:
                if completion_text:
                    return ToolResult(success=True,
                                      message=f"技能 '{skill_name}' 执行成功",
                                      content=completion_text)
                else:
                    return ToolResult(success=False,
                                      message=f"技能 '{skill_name}' 执行成功, 但未返回任何内容",
                                      content=None)
            else:
                return ToolResult(success=False,
                                  message=f"技能 '{skill_name}' 执行失败",
                                  content=completion_text)
        except Exception as e:
            return ToolResult(success=False, message=f"CallSkills 执行失败: {str(e)}",
                              content=f"错误信息: {str(e)}")

    def guide(self) -> str:
        doc = f"""
        ## call_skill (调用Skills技能)
        描述: 
        - 调用特定的技能来完成任务。技能封装了专业知识和工作流程。
        参数:
        - skill_name (必需): 要调用的技能名称。
        - request (必需): 用户的请求描述，详细说明要完成的任务。
        用法说明：
        自动匹配技能
        <call_skill>
        <skill_name>wttr</skill_name>
        <request>帮我查询武汉，上海，长沙的天气情况</request>
        </call_skill>
        """
        return doc